// Color.h
//	ShagRend3D colors and colormaps


#ifndef __Color_h__
#define __Color_h__


#include "types.h"
#include "Slope.h"


class Color {
	friend class ColorSlope;
public:
	Color()				{ red=0; green=0; blue=0; }
	Color(int v)			{ red=v; green=v; blue=v; }
	Color(int r, int g, int b)	{ red=r; green=g; blue=b; }
	ushort	red;
	ushort	green;
	ushort	blue;
	Color& operator*= (int f)
		{ red=red*f/65535; green=green*f/65535; blue=blue*f/65535;
		  return *this; }
	Color operator* (int f) { Color c;
		  c.red=int(red)*f/65535;
		  c.green=int(green)*f/65535;
		  c.blue=int(blue)*f/65535;
		  return c; }
};


class ColorBlend {
public:
	int	red;
	int	green;
	int	blue;
	int	n;
	ColorBlend()	{ red=0; green=0; blue=0; n=0; }
	ColorBlend& operator+= (Color &c)
			{ red+=c.red; green+=c.green; blue+=c.blue; n++; }
	operator Color() {
		if(n) return Color(red/n, green/n, blue/n);
		else return Color(0, 0, 0); }
};


class ColorSlope : public Color {
	uint	red_val;
	uint	green_val;
	uint	blue_val;
	int	red_inc;
	int	green_inc;
	int	blue_inc;
public:
	ColorSlope() {}
	ColorSlope(Color &c1, Color &c2, int x) {
		if(x <= 0)
			x = 1;
		red = c1.red;
		red_val = red<<16;
		if(c2.red >= c1.red)
			red_inc = ((c2.red<<16)-red_val)/x;
		else
			red_inc = -((red_val-(c2.red<<16))/x);
		green = c1.green;
		green_val = green<<16;
		if(c2.green >= c1.green)
			green_inc = ((c2.green<<16)-green_val)/x;
		else
			green_inc = -((green_val-(c2.green<<16))/x);
		blue = c1.blue;
		blue_val = blue<<16;
		if(c2.blue >= c1.blue)
			blue_inc = ((c2.blue<<16)-blue_val)/x;
		else
			blue_inc = -((blue_val-(c2.blue<<16))/x);
	}
	ColorSlope& operator++() {
		red_val += red_inc;
		red = red_val >> 16;
		green_val += green_inc;
		green = green_val >> 16;
		blue_val += blue_inc;
		blue = blue_val >> 16;
		return *this;
	}
};


class ColorMap {
	int	*pix;
	Color	*col;
	int	ncolors;
	int	nred;
	int	ngreen;
	int	nblue;
	int	ired[256];
	int	igreen[256];
	int	iblue[256];
	int	pred[256];
	int	pgreen[256];
	int	pblue[256];
	int	seed;
public:
	ColorMap(int size);
	~ColorMap();
	int	size()		{ return ncolors; }
	Color&	color(int i)	{ return col[i]; }
	int&	pixel(int i)	{ return pix[i]; }
	int	match(Color &c) {
			seed = seed*1103515245+12345;
			int qred = ired[c.red>>8]+
			 	((seed&0xff)<pred[c.red>>8]);
			seed = seed*1103515245+12345;
			int qgreen = igreen[c.green>>8]+
    				((seed&0xff)<pgreen[c.green>>8]);
			seed = seed*1103515245+12345;
			int qblue = iblue[c.blue>>8]+
    				((seed&0xff)<pblue[c.blue>>8]);
			// int qred = ired[c.red>>8];
			// int qgreen = igreen[c.green>>8];
			// int qblue = iblue[c.blue>>8];
			return pix[(qred*ngreen+qgreen)*nblue+qblue];
	}
};

#endif
