// Eye.C

#include <stdio.h>
#include "types.h"
#include "Eye.h"
#include "Obj3d.h"

Eye::Eye()
{
	w = Vector(0,0,1);
	u = Vector(0,-1,0);
	normalize();
	flength = 1.0;
	fratio = 0.75;
	frx = 1.0;
	fry = 1.0;
	spacing = 10.0;
	use_split = false;
	use_view = true;
	obj = 0;
}


void Eye::set_view(ViewPort &vp)
{
	view = &vp;
	if(vp.width() < vp.height()) 
		flength = (vp.width()/2) / fratio;
	else
		flength = (vp.height()/2) / fratio;
	frx = (vp.width()/2-1) / flength;
	fry = (vp.height()/2-1) / flength;
}


void Eye::set_transform()
{
	normalize();
	transform = Matrix4x4(u, v, w) * Matrix4x4(obj->pos);
}


// expects u and w vectors to be defined, with w pointing in 
// direction of sight, and u representing 'up'
// sets 'v' vector
void Eye::normalize()
{
	w.normalize();
	double n = -(u*w);
        if(n != 0.0)
		u *= n;
	u.normalize();
	v = w ^ u;
	v.normalize();
}



extern void dummy_func(volatile ViewPort *vp);


IVector Eye::project(Vector &v)
{
IVector i;
#ifdef DEBUG
	printf("vec = %f %f %f\n", v.x, v.y, v.z); fflush(stdout);
	dummy_func(view);
	printf("xctr,yctr=%d,%d flength=%f frx=%f fry=%f\n",
		view->xcenter(), view->ycenter(),
		flength, frx, fry); fflush(stdout);
	printf("ZCLIP_MIN=%f ZCLIP_MAX=%f ZBUF_MAX=%d\n",
		ZCLIP_MIN, ZCLIP_MAX, ZBUF_MAX); fflush(stdout);
#endif
	i.x = int(view->xcenter() + flength*v.x/v.z);
	i.y = int(view->ycenter() + flength*v.y/v.z);
	i.z = ZBUF_MIN + int((v.z-ZCLIP_MIN)
			*((ZBUF_MAX-ZBUF_MIN)/(ZCLIP_MAX-ZCLIP_MIN)));
	if(i.z < ZBUF_MIN || i.z > ZBUF_MAX || v.z < ZCLIP_MIN ||
			v.z > ZCLIP_MAX) {
		fprintf(stderr, "z=%d (%d,%d) %f (%f,%f)\n", i.z,
			ZBUF_MIN, ZBUF_MAX,
			v.z, ZCLIP_MIN, ZCLIP_MAX);
		exit(1);
	}
#ifdef DEBUG
	printf("result vec: %d, %d, %d\n", i.x, i.y, i.z);
	fflush(stdout);
#endif
	return i;
}

