// Eye.h


#ifndef __Eye_h__
#define __Eye_h__

#include "types.h"
#include "Vector.h"
#include "IVector.h"
#include "Matrix.h"
#include "ViewPort.h"


#define ZCLIP_MIN 	0.10			 // 100 cm 
#define ZCLIP_MAX	24000.0			 // 20 km 



class Eye {
public:
	Vector u;
	Vector v;
	Vector w;
	class Obj3d *obj;	// object to center on
	vec_t flength;		// focal length 
	vec_t fratio;		// focal ratio 
	vec_t frx;		// focal ratio for x 
	vec_t fry;		// focal ratio for y 
	volatile ViewPort *view;	// viewport for non-stereo views
	volatile ViewPort *left;	// viewport for left eye
	volatile ViewPort *right;	// viewport for right eye
	double spacing;		// spacing between eyes
	bool use_split;		// split screen (use left/right)
	bool use_view;		// use main viewport 
	Matrix4x4 transform;	// current transformation matrix
	void set_object(Obj3d *o) { obj = o; }
	void set_view(ViewPort &vp);	// set main viewport
	void set_transform();	// update transformation matrix
	void moveto(Vector v);	// move to vector
	void lookat(Vector v);	// turn towards vector
	void normalize();	// normalize u,w, and set v
	IVector project(Vector &v);	// project to screen coords
	Eye();
};


#endif
