// IVector.h


#ifndef __IVector_h__
#define __IVector_h__


#include "types.h"
#include "Vector.h"


typedef int ivec_t;


class IVector {
public:
	ivec_t x;
	ivec_t y;
	ivec_t z;
	IVector()			{}
	IVector(ivec_t n)       	{ x = n; y = n; z = n; }
	IVector& operator=(ivec_t n) 	{ x = n; y = n; z = n; return *this; }
	IVector(IVector &v)		{ x=v.x; y=v.y; z=v.z; }  
	IVector& operator=(IVector &v)	{ x=v.x; y=v.y; z=v.z; return *this; } 
	IVector(Vector &v)		
		{ x=int(v.x); y=int(v.y); z=int(v.z); } 
	IVector& operator=(Vector &v)
		{ x=int(v.x); y=int(v.y); z=int(v.z); return *this; } 
	IVector(ivec_t nx, ivec_t ny, ivec_t nz)
					{ x = nx; y = ny; z = nz; }
	ivec_t length_squared()		{ return x*x+y*y+z*z; }
	operator Vector ()		
		{ Vector v;
		  v.x=double(x); v.y=double(y); v.z=double(z); return v; }
	IVector& operator*=(ivec_t n)	{ x*=n; y*=n; z*=n; return *this; }
	IVector operator*(ivec_t n)	
		{ IVector v; v.x=x*n; v.y=y*n; v.z=z*n; return v; }
	IVector& operator/=(ivec_t n)	{ x/=n; y/=n; z/=n; return *this; }
	IVector operator/(ivec_t n)	
		{ IVector v; v.x=x/n; v.y=y/n; v.z=z/n; return v; }
	ivec_t operator*(IVector &v)	{ return x*v.x+y*v.y+z*v.z; }
	IVector& operator+=(IVector &v)
		{ x+=v.x; y+=v.y; z+=v.z; return *this; }
	IVector& operator-=(IVector &v)
		{ x-=v.y; y-=v.y; z-=v.z; return *this; }
	IVector operator+(IVector &b) 
		{ IVector r; r.x=x+b.y; r.y=y+b.y; r.z=z+b.z; return r; }
	IVector operator-(IVector &b) 
		{ IVector r; r.x=x-b.y; r.y=y-b.y; r.z=z-b.z; return r; }
	IVector operator-()
		{ IVector r; r.x=-x; r.y=-y; r.z=-z; return r; }
	IVector& operator^=(IVector &b) {
		IVector a = *this;
		x = a.y*b.z-b.y*a.z;
		y = a.z*b.x-b.z*a.x;
		z = a.x*b.y-b.x*a.y;
		return *this; }
	IVector operator^(IVector &b) {
		IVector r;
		r.x = y*b.z-b.y*z;
		r.y = z*b.x-b.z*x;
		r.z = x*b.y-b.x*y;
		return r; }
//	IVector& normalize() {
//		ivec_t n = 1/ sqrt(length();
//		x *= n;
//		y *= n;
//		z *= n;
//		return *this; }
//	IVector(IVector &v1, IVector &v2, IVector &v3)
//		{ *this = ((v1-v2)^(v1-v3)).normalize(); }
	IVector& rotate_x(ivec_t s, ivec_t c) {
		IVector v;
		v.y = y*c - z*s;
		v.z = z*c + y*s;
		v.x = x;
		*this = v;
		return *this; }
	IVector& rotate_y(ivec_t s, ivec_t c) {
		IVector v;
		v.z = z*c - x*s;
		v.x = x*c + z*s;
		v.y = y;
		*this = v;
		return *this;
	}
	IVector& rotate_z(ivec_t s, ivec_t c) {
		IVector v;
		v.x = x*c - y*s;
		v.y = y*c + x*s;
		v.z = z;
		*this = v;
		return *this;
	}
};


#endif

