// Matrix.h

#ifndef __Matrix_h__
#define __Matrix_h__


#include <sys/types.h>

#include "types.h"
#include "Vector.h"
#include "IVector.h"
#include "math.h"



class Matrix4x1 {
public:
	vec_t	m[4];
	Matrix4x1(vec_t n = 0) { m[0] = n; m[1] = n; m[2] = n; m[3] = n; }
	Matrix4x1& operator=(vec_t n)
		{ m[0] = n; m[1] = n; m[2] = n; m[3] = n; return *this; }
	Matrix4x1(Vector &v) { m[0] = v.x; m[1] = v.y; m[2] = v.z; m[3] = 0; }
	Matrix4x1(IVector &v) { m[0] = v.x; m[1] = v.y; m[2] = v.z; m[3] = 0; }
	Matrix4x1& operator=(Vector &v)
		{ m[0] = v.x; m[1] = v.y; m[2] = v.z; m[3] = 0; return *this; }
	Matrix4x1& operator=(IVector &v)
		{ m[0] = v.x; m[1] = v.y; m[2] = v.z; m[3] = 0; return *this; }
	vec_t& operator[](int n) { return m[n]; }
};


class Matrix4x4 {
public:
	Matrix4x1 m[4];

	Matrix4x4(vec_t n = 0) { m[0] = n; m[1] = n; m[2] = n; m[3] = n; }
	Matrix4x4& operator=(vec_t n)
		{ m[0] = n; m[1] = n; m[2] = n; m[3] = n; return *this; }
	Matrix4x4(Vector &v) {
		*this = 0;
		m[0][0]=1; m[1][1]=1; m[2][2]=1; m[3][3]=1;
		m[0][3]=-v.x; m[1][3]=-v.y; m[2][3]=-v.z; }
	Matrix4x4(IVector &v) {
		*this = 0;
		m[0][0]=1; m[1][1]=1; m[2][2]=1; m[3][3]=1;
		m[0][3]=-v.x; m[1][3]=-v.y; m[2][3]=-v.z; }
	Matrix4x4& operator= (Vector &v) {
		*this = 0;
		m[0][0]=1; m[1][1]=1; m[2][2]=1; m[3][3]=1;
		m[0][3]=-v.x; m[1][3]=-v.y; m[2][3]=-v.z; return *this; }
	Matrix4x4& operator= (IVector &v) {
		*this = 0;
		m[0][0]=1; m[1][1]=1; m[2][2]=1; m[3][3]=1;
		m[0][3]=-v.x; m[1][3]=-v.y; m[2][3]=-v.z; return *this; }
	Matrix4x4(Vector &u, Vector &v, Vector &w) {
		m[0] = v; m[1] = u; m[2] = w; m[3] = 0; }
	Matrix4x4(IVector &u, IVector &v, IVector &w) {
		m[0] = v; m[1] = u; m[2] = w; m[3] = 0; }
	Matrix4x1& operator[](int n) { return m[n]; }
	Matrix4x4& identity() {
		*this = 0;
		m[0][0] = m[1][1] = m[2][2] = m[3][3] = 1.0; }
};


inline Matrix4x1 operator* (Matrix4x1 &a, Matrix4x4 &b) {
	Matrix4x1 r;
	r[0] =   a[0]*b[0][0]+a[1]*b[1][0]
		+a[2]*b[2][0]+a[3]*b[3][0];
	r[1] =   a[0]*b[0][1]+a[1]*b[1][1]
		+a[2]*b[2][1]+a[3]*b[3][1];
	r[2] =   a[0]*b[0][2]+a[1]*b[1][2]
		+a[2]*b[2][2]+a[3]*b[3][2];
	r[3] =   a[0]*b[0][3]+a[1]*b[1][3]
		+a[2]*b[2][3]+a[3]*b[3][3];
	return r;
}


inline Matrix4x4 operator* (Matrix4x4 &a, Matrix4x4 &b) {
	Matrix4x4 r;
	r[0] =	a[0]*b;
	r[1] =	a[1]*b;
	r[2] =	a[2]*b;
	r[3] =	a[3]*b;
	return	r;
}



inline Vector operator* (Vector &v, Matrix4x4 &m)
{
Vector r;
	r.x = m[0][0]*v.x+m[0][1]*v.y+m[0][2]*v.z+m[0][3];
	r.y = m[1][0]*v.x+m[1][1]*v.y+m[1][2]*v.z+m[1][3];
	r.z = m[2][0]*v.x+m[2][1]*v.y+m[2][2]*v.z+m[2][3];
	return r;
}


inline Vector operator* (IVector &iv, Matrix4x4 &m)
{
Vector v;
Vector r;
	v.x = double(iv.x);
	v.y = double(iv.y);
	v.z = double(iv.z);
	r.x = m[0][0]*v.x+m[0][1]*v.y+m[0][2]*v.z+m[0][3];
	r.y = m[1][0]*v.x+m[1][1]*v.y+m[1][2]*v.z+m[1][3];
	r.z = m[2][0]*v.x+m[2][1]*v.y+m[2][2]*v.z+m[2][3];
	return r;
}


class Matrix_Rotate_x : public Matrix4x4 {
public:
	Matrix_Rotate_x(angle_t r) {
		*(Matrix4x4*)this = 0;
		vec_t sinr = dsin(r);
		vec_t cosr = dcos(r);
		m[0][0] = m[3][3] = 1.0;
		m[1][1] = m[2][2] = cosr;
		m[1][2] = sinr; m[2][1] = -sinr; }
//	1.0
//		cos	-sin
//		sin	cos
//				1.0
};
 
class Matrix_Rotate_y : public Matrix4x4 {
public:
	Matrix_Rotate_y(angle_t r) {
		*(Matrix4x4*)this = 0;
		vec_t sinr = dsin(r);
		vec_t cosr = dcos(r);
		m[1][1] = m[3][3] = 1.0;
		m[2][2] = m[0][0] = cosr;
		m[2][0] = sinr; m[0][2] = -sinr; } 
//	cos		sin
//		1.0
//	-sin		cos
//				1.0
};

class Matrix_Rotate_z : public Matrix4x4 {
public:
	Matrix_Rotate_z(angle_t r) {
		*(Matrix4x4*)this = 0;
		vec_t sinr = dsin(r);
		vec_t cosr = dcos(r);
		m[2][2] = m[3][3] = 1.0;
		m[0][0] = m[1][1] = cosr;
		m[0][1] = sinr; m[1][0] = -sinr; }
//	cos	-sin
//	sin	cos
//			1.0
//				1.0
};

class Matrix_Rotate_axis : public Matrix4x4 {
public:
	Matrix_Rotate_axis(Matrix4x4 &m) : Matrix4x4(m) {}
	Matrix_Rotate_axis(Vector &a, Vector &d, angle_t r) {
		Matrix4x4 f(a);
		Matrix4x4 fi(-a);
		angle_t n = datan2(d.x, d.y);
		Matrix_Rotate_z g(n);
		Matrix_Rotate_z gi(-n);
		n = datan2(d.z,sqrt(d.x*d.x+d.y*d.y));
		Matrix_Rotate_y h(n);
		Matrix_Rotate_y hi(-n);
		Matrix_Rotate_z w(r);
		*this = fi*(gi*(hi*(w*(h*(g*f))))); }
};


#endif

