// Membuf.C

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include "Membuf.h"
#include "types.h"


void Membuf::error()
{
	fprintf(stderr, "Unexpected EOF\n");
	exit(1);
}


int Membuf::read(void *adr, int size) {
int tot = 0;
int n;
	while(size--) {
		if(rpos == wpos) {
			n = read_block(&buf[wpos], bufsize-wpos);
			if(n <= 0) {
				*((uchar*&)adr)++ = 0;  // implied QUIT 
				return 0;
			} else
				wpos += n;
			if(wpos >= bufsize)
				wpos = 0;
		}
		*((uchar*&)adr)++ = buf[rpos++];
		if(rpos >= bufsize)
			rpos = 0;
		tot++;
	}
	return tot;
}


int Membuf::write(void *adr, int size) {
int n = 0;
	while(size--) {
		buf[wpos++] = *((uchar*&)adr)++;
		if(wpos >= bufsize)
			wpos = 0;
		if(wpos == rpos) {
			rpos += write_block(&buf[rpos], bufsize-rpos);
			if(rpos >= bufsize)
				rpos = 0;
		}
		n++;
	}
	return n;
}


void Membuf::flush() {
	while(wpos != rpos) {
		if(rpos > wpos) 
			rpos += write_block(&buf[rpos], bufsize-rpos);
		else 
			rpos += write_block(&buf[rpos], wpos-rpos);
		if(rpos >= bufsize)
			rpos = 0;
	}
}


bool Membuf::can_read() {
	if(rpos != wpos)
		return true;
	else
		return can_read_block();
}


int Membuf_fd::read_block(void *adr, int size)
{
	return ::read(fd, adr, size);
}


int Membuf_fd::write_block(void *adr, int size)
{
	return ::write(fd, adr, size);
}


bool Membuf_fd::can_read_block()
{
fd_set rdy;
struct timeval tv;
	FD_ZERO(&rdy);
	FD_SET(fd, &rdy);
	tv.tv_sec = 0;
	tv.tv_usec = 0;
	select(fd+1, &rdy, 0, 0, &tv);
	return FD_ISSET(fd, &rdy);
}


Membuf_fd::Membuf_fd(int newfd, int size)
	: Membuf((buf = new uchar[size]), size)
{
	fd = newfd;
}


Membuf_fd::Membuf_fd(char *fname, int size) 
	: Membuf((buf = new uchar[size]), size)
{
	if(!(fd = open(fname, O_RDONLY))) {
		perror(fname);
		exit(1);
	}
}


Membuf_fd::~Membuf_fd()
{
	delete[] buf;
}

