// Off3d.C

#include <stdio.h>
#include "types.h"
#include "Poly3d.h"
#include "Membuf.h"
#include "Off3d.h"


static int read_int(Membuf *mb)
{
char ch;
int n = 0;
	while(mb->read(ch), ch < '0' || ch > '9')
		if(ch == '#')
			while(mb->read(ch), ch != '\n')
				continue;
	while(ch >= '0' && ch <= '9') {
		n = n*10 + ch - '0';
		mb->read(ch);
	}
	return n;
}


static double read_double(Membuf *mb)
{
char ch;
double n = 0;
double div = 0.0;
double sign = 1.0;
	while(mb->read(ch), (ch < '0' || ch > '9') && ch != '-' && ch != '.')
		if(ch == '#')
			while(mb->read(ch), ch != '\n')
				continue;
	while((ch >= '0' && ch <= '9') || ch == '.' || ch == '-') {
		if(ch == '-')
			sign = -sign;
		else if(ch == '.') 
			div = 1.0;
		else {
			n = n*10.0 + (ch - '0');
			div *= 10.0;
		}
		mb->read(ch);
	}
	if(div < 1.0)
		div = 1.0;
	return n/div*sign;
}


Off3d::Off3d(int id, int scale, Membuf *mb, Membuf *cmb) : Poly3d(id)
{
int p;
	printf("loading off (%x,%x)\n", (int)mb, (int)cmb); fflush(stdout);
	npoints = read_int(mb);
	npoly = read_int(mb); 
	int nvert = read_int(mb);
	((Poly3d*)this)->Poly3d(id, npoints, npoly, nvert);
	for(p=0; p<npoints; p++) {
		vert[p].x = read_double(mb)*scale;
		vert[p].y = read_double(mb)*scale;
		vert[p].z = read_double(mb)*scale;
	}
	ushort *pp = poly[0];
	for(p=0; p<npoly; p++) {
		polysize[p] = read_int(mb);
		poly[p] = pp;
		for(int v=0; v<polysize[p]; v++) 
			*pp++ = read_int(mb)-1;
		
	}
	printf("npoints=%d npoly=%d nvert=%d\n",
		npoints, npoly, nvert); fflush(stdout);
	if(cmb && cmb->ok()) {
		int ncolors = read_int(cmb);
		if(read_int(cmb) != npoly) {
			fprintf(stderr, "Error in off color file\n");
			exit(1);
		}
		Color *col = new Color[ncolors];
		for(p=0; p<ncolors; p++)
			col[p] = Color(
				(int)(read_double(cmb)*65535),
				(int)(read_double(cmb)*65535),
				(int)(read_double(cmb)*65535));
		for(p=0; p<npoly; p++)
			color[p] = col[read_int(cmb)-1];
		delete col;
	} else {
		for(p=0; p<npoly; p++)
			color[p] = Color(65535,65535,65535);
	}
	printf("color blending\n");
	for(int v=0; v<npoints; v++) {
		printf("v=%d\n", v);
		ColorBlend c;
		for(int p=0; p<npoly; p++) 
			for(int n=0; n<polysize[p]; n++)
				if(poly[p][n] == v)
					c += color[p];
		vert[v] = (Color)c;
		printf("vert(%d): %f,%f,%f (%d,%d,%d)\n",
			v, vert[v].x, vert[v].y, vert[v].z,
			vert[v].red, vert[v].green, vert[v].blue);
		fflush(stdout);
	}
	printf("done loading\n"); fflush(stdout);
};

