// Poly3d.C


#include <stdio.h>
#include <stdlib.h>
#include "types.h"
#include "Color.h"
#include "Vector.h"
#include "Vertex.h"
#include "IVertex.h"
#include "Matrix.h"
#include "Obj3d.h"
#include "Poly3d.h"
#include "Eye.h"


Poly3d::Poly3d(int id, int points, int polys, int nvert) : Obj3d(id)
{
	npoints = points;
	npoly = polys;
	vert = new Vertex[npoints];
	trans = new Vertex[npoints];
	color = new Color[npoly];
	polysize = new ushort[npoly];
	poly = new (ushort*)[npoly];
	poly[0] = new (ushort)[nvert];
	smooth = false;
}

Poly3d::~Poly3d()
{
	delete[] vert;
	delete[] trans;
	delete[] color;
	delete[] polysize;
	delete[] poly[0];
	delete[] poly;
}



static inline bool clip_t(double df, double nf, double &tmin, double &tmax)
{
double t;
	if(df > 0.0) {
		t = nf/df;		
		if(t > tmax)
			return false;
		else if(t > tmin)	 
			tmin = t;
	} else if(df < 0.0) {
		t = nf/df;
		if(t < tmin)
			return false;
		else if(t < tmax)
			tmax = t;
	} else if(nf > 0.0)
		return false;
	return true;
}


static int clip_line(Vector &v0, Vector &v1, double fx, double fy)
{
double tmin = 0.0;
double tmax = 1.0;
Vector d = v1 - v0;
#ifdef DEBUG
	printf("clip: (%f,%f,%f)-(%f,%f,%f) ",
		v0.x, v0.y, v0.z,  v1.x, v1.y, v1.z);
	fflush(stdout);
#endif
	if(   !clip_t(d.x+d.z*fx, -v0.x-fx*v0.z, tmin, tmax)
	   || !clip_t(-d.x+d.z*fx, v0.x-fx*v0.z, tmin, tmax)
	   || !clip_t(-d.y+d.z*fy, v0.y-fy*v0.z, tmin, tmax)
	   || !clip_t(d.y+d.z*fy, -v0.y-fy*v0.z, tmin, tmax)
	   || !clip_t(d.z, -v0.z+ZCLIP_MIN, tmin, tmax)
	   || !clip_t(-d.z, v0.z-ZCLIP_MAX, tmin, tmax))
		return -1;
	if(tmax < 1.0) 
		v1 = v0 + Vector(tmax*d.x, tmax*d.y, tmax*d.z);
	if(tmin > 0.0) 
		v0 += Vector(tmin*d.x, tmin*d.y, tmin*d.z);
#ifdef DEBUG
	printf(" to: (%f,%f,%f)-(%f,%f,%f)\n",
		v0.x, v0.y, v0.z,  v1.x, v1.y, v1.z);
	fflush(stdout);
	printf("tmin=%f tmax=%f\n", tmin, tmax);
	fflush(stdout);
	printf("d=%f,%f,%f\n", d.x, d.y, d.z);
	fflush(stdout);
#endif
	if(tmin < 0.0 || tmin > 1.0 || tmax < 0.0 || tmax > 1.0)
		exit(1);
	fflush(stdout);
	return((tmax < 1.0) || (tmin > 0.0));
}




static int clip_polygon_viewport(Vertex *p, int psides, double fx, double fy)
{
	for(int c=0; c<6; c++) {
		Vertex r[MAXSIDES];
		int nr = 0;
		for(int n=0; n<psides; n++) {
			int n2 = n+1;
			if(n2 >= psides)
				n2 = 0;
			Vector d = p[n2] - p[n];
			double tmin = 0.0;
			double tmax = 1.0;
#ifdef DEBUG
			printf("VECTOR: (%f,%f,%f)-(%f,%f,%f) d=(%f,%f,%f)\n",
				p[n].x, p[n].y, p[n].z,
				p[n2].x, p[n2].y, p[n2].z,
				d.x, d.y, d.z);
			fflush(stdout);
#endif
			switch(c) {
			case 0: if(!clip_t(d.x+d.z*fx, -p[n].x-fx*p[n].z,
						tmin,tmax))
					continue;
				break;
			case 1:	if(!clip_t(-d.x+d.z*fx, p[n].x-fx*p[n].z,
						tmin,tmax))
					continue;
				break;
			case 2: if(!clip_t(-d.y+d.z*fy, p[n].y-fy*p[n].z,
						tmin,tmax))
					continue;
				break;
			case 3: if(!clip_t(d.y+d.z*fy, -p[n].y-fy*p[n].z,
						tmin,tmax))
					continue;
				break;
			case 4: if(!clip_t(d.z, -p[n].z+ZCLIP_MIN,
						tmin,tmax))
					continue;
				break;
			case 5: if(!clip_t(-d.z, p[n].z-ZCLIP_MAX,
						tmin,tmax))
					continue;
			}
			if(tmin > 0.0) {
				r[nr] = (Color)p[n];
				r[nr].x=p[n].x+tmin*d.x;
				r[nr].y=p[n].y+tmin*d.y;
				r[nr].z=p[n].z+tmin*d.z;
#ifdef DEBUG
				printf("r[%d]: (%f,%f,%f) (tmax=%f)\n",
					nr, r[nr].x, r[nr].y, r[nr].z, tmin);
				fflush(stdout);
#endif
				nr++;
			}
			r[nr] = (Color)p[n];
			r[nr].x=p[n].x+tmax*d.x;
			r[nr].y=p[n].y+tmax*d.y;
			r[nr].z=p[n].z+tmax*d.z;
#ifdef DEBUG
			printf("r[%d]: (%f,%f,%f) (tmax=%f)\n",
				nr, r[nr].x, r[nr].y, r[nr].z, tmax);
			fflush(stdout);
#endif
			nr++;
		}
		psides = nr;
		for(n=0; n<psides; n++)
			p[n] = r[n];
	}
	return psides;
}


static void render_polygon(Eye &e, Vertex *v,
				int nsides, Vector &light)
{
IVertex iv[MAXSIDES];
Vertex p,p2;
	if(!e.view->wireframe_set()) {
#ifdef DEBUG
		printf("before clipping:\n");
		fflush(stdout);
		for(int i=0; i<nsides; i++)
			printf(" v[%d]: %f,%f,%f\n", i,
				v[i].x, v[i].y, v[i].z);
#endif
		int ns = clip_polygon_viewport(v,nsides,e.frx,e.fry);
#ifdef DEBUG
		printf("after clipping:\n");
		fflush(stdout);
		for(i=0; i<ns; i++)
			printf(" v[%d]: %f,%f,%f\n", i,
				v[i].x, v[i].y, v[i].z);
#endif
		if(ns < 3)
			return;
		for(int n=0; n<ns; n++) {
			iv[n] = (Color)v[n];
			iv[n] = e.project(v[n]);
		}
#ifdef DEBUG
		printf("fill polygon:\n");
		fflush(stdout);
		for(i=0; i<ns; i++)
			printf(" v[%d]: %d,%d,%d\n", i,
				iv[i].x, iv[i].y, iv[i].z);
#endif
		e.view->fill_polygon(iv, ns);
#ifdef DEBUG
		printf("fill done\n");
		fflush(stdout);
#endif
	} else {
		int ns = 0;
		for(int n=0; n<nsides; n++) {
			p = v[n];	
			if(!n)
				p2 = v[nsides-1];
			else
				p2 = v[n-1];
#ifdef DEBUG
			printf("clipping: (%f,%f,%f)-(%f,%f,%f)\n",
					p.x,p.y,p.z, p2.x,p2.y,p2.z);
			fflush(stdout);
#endif
			switch(clip_line(p,p2,e.frx, e.fry)) {
			case -1:
				continue;
			case true:
				iv[ns++].z = -1;
				iv[ns] = (Color)p2;
				iv[ns++] = e.project(p2);
			default:
				;
			}
			iv[ns] = (Color)p;
			iv[ns++] = e.project(p);
		}
#ifdef DEBUG
		printf("draw polygon\n");
		fflush(stdout);
#endif
		e.view->draw_polygon(iv, ns);
#ifdef DEBUG
		printf("done polygon\n");
		fflush(stdout);
#endif
	}
}


 
void Poly3d::render(Eye &e, Vector &light)
{
Matrix4x4 xform = translate(e.transform);
int i;
#ifdef DEBUG
	printf("render\n"); 
	fflush(stdout);
#endif
	for(i=0; i<npoints; i++) {
#ifdef DEBUG
		printf("print vector:\n"); fflush(stdout);
		fflush(stdout);
		printf("v: %f,%f,%f\n", 
			vert[i].x, vert[i].y, vert[i].z);
		fflush(stdout);
		fflush(stdout);
#endif
		trans[i] = (Color)vert[i];
		trans[i] = (Vector)vert[i] * xform;
#ifdef DEBUG
		printf("t: %f,%f,%f\n",
			trans[i].x, trans[i].y, trans[i].z);
		fflush(stdout);
#endif
	}
#ifdef DEBUG
	printf("trns\n"); fflush(stdout);
#endif
	for(i=0; i<npoly; i++) {
		int nsides = polysize[i];
		Vertex v[MAXSIDES];
		if(nsides < 3) {
			fprintf(stderr,
			"Warning: polygon %d of object %x:%x has %d sides\n",
				i, (int)this, id, nsides);
		}
		int n;
#ifdef DEBUG
		printf("Polygon %d:\n", i);
		fflush(stdout);
#endif
		for(n=0; n<nsides; n++)  {
			v[n] = trans[poly[i][n]];
#ifdef DEBUG
			printf(" vec: %f,%f,%f\n", v[n].x,v[n].y,v[n].z);
			fflush(stdout);
#endif
		}
#ifdef DEBUG
		for(n=0; n<npoints; n++)
			printf("t(%d): %f,%f,%f\n", n, 
				trans[n].x, trans[n].y, trans[n].z);
		fflush(stdout);
// do color translation and take the normal to the plane here
		printf("poly render\n"); fflush(stdout);
#endif
		if(nsides < 3)
			continue;
		Vector nv = Vector(v[0],v[1],v[2]);
		int lf = int((nv*light)*16384+32768+16384);
		if(!e.view->gourad_set())
			for(n=0; n<nsides; n++) 
				v[n] = (Color)color[i] * lf;
		else	
			for(n=0; n<nsides; n++) 
				v[n] = (Color)trans[poly[i][n]] * lf;
		render_polygon(e, v, nsides, light);
	}
};

