// Screen_svga.h
//	ShagRend3D screen specifications

#ifndef __Screen_h__
#define __Screen_h__


#include "types.h"
#include "Color.h"
#include "ioport.h"
#include "svga.h"

typedef unsigned char Pixel;


struct mouse_state {
	int	oldx, oldy;
	int	x, y;
	bool	button[3];
	bool	oldbutton[3];
};


class Screen {
	int	wid, hgt;
	ColorMap *cmap;
	Pixel	*vidmem;
	uchar	*font;
	int	vpitch;
	int	framesize;
	static int page;
	static int planemask;
	static int vbase;			// base address for next page to show
	static int vofs;			// current video offset (displayed)
public:
	Screen();
	~Screen();
	static int pagebase[4];
	int width()	{ return wid; }
	int height()	{ return hgt; }
	ColorMap* colormap() { return cmap; }
	Pixel* video_base()  { return vidmem; }
	int get_key();
	bool read_mouse(mouse_state *state);
	void write(int page, int x, int y, char *s, int c0, int cinc);
	void write(int page, int x, int y, char *s)
		{ write(page, x, y, s, 251, 0); }
	void write(int x, int y, char *s, int c0, int cinc)
		{ write(page, x, y, s, c0, cinc); }
	void write(int x, int y, char *s)
		{ write(page, x, y, s, 251, 0); }
	void write_all(int x, int y, char *s, int c0, int cinc) {
		write(0, x, y, s, c0, cinc);
		write(1, x, y, s, c0, cinc);
		write(2, x, y, s, c0, cinc); }
	void write_all(int x, int y, char *s)
		{ write(x, y, s, 251, 0); }
	static void set_mask(register int mask) {
		if(mask != planemask) {
			planemask = mask;
			outb(SEQ_I, 0x02);
			outb(SEQ_D, mask);
		}
	}
	static void set_vofs(int adr) {
		inb(IS1_RC);
		outw(CRT_IC, 0xd + ((adr & 0xff) << 8));
		outw(CRT_IC, 0xc + ((adr & 0xff00)));
	}
	static int flip_page() {
		set_vofs((vofs = pagebase[page]));
		if(++page > 2)  page = 0;
		vbase = pagebase[page];
		return page;
	}
};

#endif
