// Vector.h


#ifndef __Vector_h__
#define __Vector_h__


#include <stdio.h>
#include "types.h"
#include "math.h"


typedef double vec_t;


class Vector {
public:
	vec_t x;
	vec_t y;
	vec_t z;
	Vector()			{}
	Vector(vec_t n)       		{ x = n; y = n; z = n; }
	Vector& operator=(vec_t n) 	{ x = n; y = n; z = n; return *this; }
	Vector(Vector &v)		{ x=v.x; y=v.y; z=v.z; }  
	Vector& operator=(Vector &v)	{ x=v.x; y=v.y; z=v.z; return *this; } 
	Vector(vec_t nx, vec_t ny, vec_t nz)
					{ x = nx; y = ny; z = nz; }
	vec_t length()			{ return dsqrt(x*x+y*y+z*z); }
	Vector& operator*=(vec_t n)	{ x*=n; y*=n; z*=n; return *this; }
	Vector operator*(vec_t n)
		{ Vector r; r.x=x*n; r.y=y*n; r.z=z*n; return r; }
	Vector& operator/=(vec_t n)	{ x/=n; y/=n; z/=n; return *this; }
	Vector operator/(vec_t n)
		{ Vector r; r.x=x/n; r.y=y/n; r.z=z/n; return r; }
	vec_t operator*(Vector &v)	{ return x*v.x+y*v.y+z*v.z; }
	Vector& operator+=(Vector &v)
		{ x+=v.x; y+=v.y; z+=v.z; return *this; }
	Vector& operator-=(Vector &v)
		{ x-=v.x; y-=v.y; z-=v.z; return *this; }
	Vector operator+(Vector &b) 
		{ Vector r; r.x=x+b.x; r.y=y+b.y; r.z=z+b.z; return r; }
	Vector operator-(Vector &b) 
		{ Vector r; r.x=x-b.x; r.y=y-b.y; r.z=z-b.z; return r; }
	Vector operator-()
		{ Vector r; r.x=-x; r.y=-y; r.z=-z; return r; }
	Vector& operator^=(Vector &b) {
		Vector a = *this;
		x = a.y*b.z-b.y*a.z;
		y = a.z*b.x-b.z*a.x;
		z = a.x*b.y-b.x*a.y;
		return *this; }
	Vector operator^(Vector &b) {
		Vector r;
		r.x = y*b.z-b.y*z;
		r.y = z*b.x-b.z*x;
		r.z = x*b.y-b.x*y;
		return r; }
	Vector& normalize() {
		vec_t n = 1.0 / length();
		x *= n;
		y *= n;
		z *= n;
		return *this; }
	Vector(Vector &v1, Vector &v2, Vector &v3)
		{ *this = ((v1-v2)^(v1-v3)).normalize(); }
	Vector& rotate_x(vec_t s, vec_t c) {
		Vector v;
		v.y = y*c - z*s;
		v.z = z*c + y*s;
		v.x = x;
		*this = v;
		return *this; }
	Vector& rotate_y(vec_t s, vec_t c) {
		Vector v;
		v.z = z*c - x*s;
		v.x = x*c + z*s;
		v.y = y;
		*this = v;
		return *this;
	}
	Vector& rotate_z(vec_t s, vec_t c) {
		Vector v;
		v.x = x*c - y*s;
		v.y = y*c + x*s;
		v.z = z;
		*this = v;
		return *this;
	}
};


#endif

