
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include "types.h"
#include "ioport.h"
#include "Vfx1.h"
#include "Vfx1Event.h"

// VIP Internal registers 
#define	VIP_SCR		 0
#define	VIP_SSR		 1
#define	VIP_ICR		 2
#define	VIP_ISR		 3
#define	VIP_PSR		 4
#define	VIP_VRR		 5
#define	VIP_PCR		 6
#define	VIP_PPMR	 7
#define	VIP_PRAR	 8
#define	VIP_PWAR	 9
#define	VIP_PCRR	10


Vfx1 vfx1;


Vfx1::Vfx1() {
	vip_air_port = 0x260;
	vip_acr_port = 0x261;
	vip_fir_port = 0x262;
	vip_fcr_port = 0x263;
	if(ioperm(vip_air_port, 4, true) < 0) 
		fprintf(stderr, "Cannot get VIP/VFX1 I/O permissions\n");
	if((devfd = open("/dev/vfx1", O_RDONLY)) < 0)
		perror("/dev/vfx1");
}


void Vfx1::video_init(bool stereo) {
uchar b;
	outb(0x3d4, 0x06);
	b = (0x01 & inb(0x3d5));
	outb(0x3d4, 0x16);
	b ^= (0x01 & inb(0x3d5));
	outb(vip_fir_port, 0x02);
	if(b) b = 0; else b = 8;
	if(stereo)
		outb(vip_fcr_port, b);
	else
		outb(vip_fcr_port, b|1);
	set_ihdelay(0);
	set_ohdelay(1);
	set_page(0);
}


void Vfx1::set_palette(int i, int red, int green, int blue)
{
int bits;
	outb(vip_air_port, VIP_PPMR);
	outb(vip_acr_port, 0xff);
	outb(vip_air_port, VIP_PCR);
	bits = inb(vip_acr_port);
	// bit 0 = snoop disable, 1 = left (right if 0), 2 = both
	// might need to select individually for stereoscopic?
	bits = (bits&~7)|0x4;
	outb(vip_acr_port, bits);
	outb(vip_air_port, VIP_PWAR);
	outb(vip_acr_port, i);
	outb(vip_air_port, VIP_PCRR);
	outb(vip_acr_port, red);
	outb(vip_acr_port, green);
	outb(vip_acr_port, blue);
}

bool Vfx1::hmd_puck_update()
{
	return read(devfd, &vfx, sizeof(vfx)) == sizeof(vfx);
}


