#ifndef _Vfx1Event_h_
#define _Vfx1Event_h_


#include "types.h"


/*
 * SHAGGY: you'll be interested in the VFXHmdEvent structure, and the
 *	   VFXEvent structure (see below)
 *
 */

/* ioctl commands */ /* XXX Tad: this doesn't currently work! */
#define VFX1_RESET_DEVICES	1


#ifdef __KERNEL__
/* abus device switch addresses */
#define ABUS_NEWDEV_ADDR	0x6e
#define ABUS_PUCK_ADDR		0x02
#define ABUS_HMD_ADDR		0x04

/* abus opcodes */
#define ABUS_ATTENTION		0xe0
#define ABUS_IDREQUEST		0xf1
#define ABUS_IDREPLY		0xe1
#define ABUS_ASSIGNADDR		0xf2
#define ABUS_CAPABLEREQ		0xf3
#define ABUS_CAPABLEREPLY	0xe3
#define ABUS_APPREPORT		0xf5
#define ABUS_POWERMODE		0xf6
#define ABUS_SETINTERVAL	0x82
#define ABUS_PUCKLOCREPORT	0x00
#define ABUS_HMDLOCREPORT	0x1d

#endif /* __KERNEL__ */


typedef enum {VFXOrientationInfo, VFXDebugInfo} VFXEventType;

 /*
  * This structure contains the orientation info from the HMD
  *
  */

typedef struct {
    short x;
    short y;
    short pitch;
    short roll;
} VFXHmdEvent;

 /*
  * This structure contains the roll, pitch, and button data from the CyberPuck
  *
  */
typedef struct {
    short roll;
    short pitch;
    uchar buttons;
} VFXPuckEvent;


/*
 * misc debugging info that could be interesting to see
 *
 */

typedef struct {
    int num_buffer_overruns;
    int num_interrupts;
    int num_puck_messages;
    int num_hmd_messages;
    int msglen;
    uchar msg[128];
} VFXDebugEvent;

 /*
  * SHAGGY: READ THIS --v
  *
  * Structure that contains the event received from reading /dev/vfx1
  * To get events from the HMD or cyberpuck, you should:
  *
  *	VFXEvent ev;
  * 	int fd = open("/dev/vfx1");
  *
  *	read(fd,&ev,sizeof(ev));
  *	if read returns 0: there is nothing to read
  *	if read returns -1: there is an error of some sort
  *	
  *	a successful return from read() will return the size of the VFXEvent
  *	so you can compare to make sure that it's a valid read.
  *	the "evtype" field will be set to one of the "VFXEventType"'s
  *	listed above:
  *		
  *		evtype == VFXOrientationInfo
  *		    This means that the VFXEvent.hmd *AND*
  *		    the VFXEvent.puck members will both contain valid data.
  *
  *		evtype == VFXDebugInfo
  *		    This means that the VFXEvent.debug member will contain
  *		    valid data (and it implies that hmd & puck are undefined)
  *
  */

typedef struct {
    VFXEventType evtype;

    VFXHmdEvent hmd;
    VFXPuckEvent puck;

    VFXDebugEvent debug;
} VFXEvent;

#endif /* _Vfx1_h_ */
