// ViewPort.C


#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <limits.h>
#include "types.h"
#include "Screen.h"
#include "ViewPort.h"
#include "ioport.h"
#include "svga.h"



void dummy_func(volatile ViewPort *vp)
{
}




int ViewPort::fps_x1000()
{
static int flast;
static struct timeval tlast;
struct timeval tv;
int tdif;
int fps_x1000;
	gettimeofday(&tv,NULL);
	tdif = (tv.tv_sec-tlast.tv_sec)*1000000
		+(tv.tv_usec-tlast.tv_usec);
	fps_x1000 = (long long)(fcount-flast)
			*(long long)(1000*1000000)/(long long)tdif;
	tlast = tv;
	flast = fcount;
	return fps_x1000;
}



void ViewPort::init()
{
	xctr = wid/2;
	yctr = hgt/2;
	pitch = screen->width();
#ifdef STEREO
	vpitch = pitch;
#else
	vpitch = pitch/4;
#endif
	drawmode = ZBUF_GOURAD_DITHER;
	fcount = 0;
	for(int i=0; i<4; i++)
		frame_yb[i] = new (Pixel*)[hgt];
	zbuf = new ZPixel[wid*hgt];
	zbuf_y = new (ZPixel*)[hgt];
	zbuf_size = wid*hgt*sizeof(ZPixel);
	cmap = screen->colormap();
	for(int y=0; y<hgt; y++) {
		for(int i=0; i<4; i++)
			frame_yb[i][y] = &(screen->video_base()
				+screen->pagebase[i])[y*vpitch];
		zbuf_y[y] = &zbuf[y*pitch];
	}
	// wait_vsync = false;
	update(); clear();
	update(); clear();
	update(); clear();
	update(); clear();
}


ViewPort::ViewPort(Screen &s, int x, int y, int w, int h)
{
	screen = &s;
	xp = x;
	yp = y;
	wid = w;
	hgt = h;
	xctr = w/2;
	yctr = h/2;
	init();
}



ViewPort::ViewPort(Screen &s)
{
	screen = &s;
	xp = yp = 0;
	wid = s.width();
	hgt = s.height();
	init();
}


ViewPort::~ViewPort()
{
int i;
Color c1, c2;
int f;
	fps_x1000();
	for(f=0; f<60; f++) {
	for(i=0; i<80; i++) {
		c1.red = i*200;
		c1.green = 200;
		c1.blue = f*1000;
		c2.red = f*300;
		c2.green = 2000;
		c2.blue = i*500;
		raw_vline(f*3+i,0,80,100,100,c1,c2);
		// raw_vline(f*3+i,0,80,50);
	}
	update();
	clear();
	}
	f = fps_x1000();
	printf("t=%d.%3d\n", f/1000, f%1000);
	for(i=0; i<4; i++)
		delete[] frame_y[i];
	delete[] zbuf;
	delete[] zbuf_y;
}


void ViewPort::update()
{
	frame_y = frame_yb[screen->flip_page()];
	fcount++;
}


void ViewPort::clear()
{
	// clear the zbuffer:
	if(drawmode & ZBUF)
		asm("rep; stosl" : :"a"(0xffffffff), "c"(zbuf_size/4),"D"(zbuf)
			:"ecx","edi");
	// set mask for all pixels:
#ifndef STEREO
	set_mask(0xf);
#endif
	// Not needed since we're triple buffering now:
	// wait for vertical sync:
	// if(wait_vsync)
	//	while(!(inb(0x3da)&8))
	//		continue; 
	// clear the frame buffer:
	Pixel **yp = &frame_y[0];
	int i = hgt;
	while(i--) {
#ifdef STEREO
		asm("rep; stosl" : :"a"(0),"c"(wid),"D"(*yp++)
			:"ecx","edi");
#else
		asm("rep; stosl" : :"a"(0),"c"(wid/4),"D"(*yp++)
			:"ecx","edi");
#endif
	}
}



// draw a polygon (don't fill)
// if for any vertex, if (v.z < 0), that vertex is not used
// and no segment is connected across it

void ViewPort::draw_polygon(IVertex iv[], int nsides)
{
int i;
Color color;
Pixel pix = 0;
//	for(i=0; i<nsides; i++) {
//		printf("polyside %d: %d,%d,%d\n", i, iv[i].x, iv[i].y, iv[i].z);
//		if((iv[i].x < 0 || iv[i].x > 319 || iv[i].y < 0 || iv[i].y
//			> 319) && iv[i].z >= 0) {
//			printf("Whoa! x,y=%d,%d\n", iv[i].x, iv[i].y);
//			exit(1);
//		}
//	}
	if(!(drawmode & GOURAD)) {
		ColorBlend c;
		for(i=0; i<nsides; i++) 
			c += (Color)iv[i];
		color = c;
		if(!(drawmode & DITHER))
			pix = cmap->match(color);
	}
	for(i=0; i<nsides; i++) {
		int n = i+1;
		if(n >= nsides)
			n = 0;
		if(iv[i].z < 0 || iv[n].z < 0)
			continue;
		switch(drawmode & (DITHER|GOURAD|ZBUF)) {
		case SOLID:
			raw_line(iv[i].x, iv[i].y, iv[n].x, iv[n].y,
					pix); break;
		case DITHER:
			raw_line(iv[i].x, iv[i].y, iv[n].x, iv[n].y,
					color); break;
		case GOURAD:
		case GOURAD_DITHER:
			raw_line(iv[i].x, iv[i].y, iv[n].x, iv[n].y,
				 iv[i], iv[n]); break;
		case ZBUF:
			raw_line(iv[i].x, iv[i].y, iv[n].x, iv[n].y,
				 iv[i].z, iv[n].z, pix); break;
		case ZBUF_DITHER:
			raw_line(iv[i].x, iv[i].y, iv[n].x, iv[n].y,
				 iv[i].z, iv[n].z, color); break;
		case ZBUF_GOURAD:
		case ZBUF_GOURAD_DITHER:
			raw_line(iv[i].x, iv[i].y, iv[n].x, iv[n].y,
				 iv[i].z, iv[n].z, iv[i], iv[n]); break;
		}
	}
}


void ViewPort::fill_polygon(IVertex iv[], int nsides)
{
short xmin = SHRT_MAX;
short xmax = SHRT_MIN;
struct {
	short	   x1;
	short	   x2;
	Slope	   y;
	Slope	   z;
	ColorSlope c;
} side[MAXSIDES];
int ns = 0;
int i;
Color color;
Pixel pix = 0;
	if(drawmode & WIREFRAME) {
		draw_polygon(iv, nsides);
		return;
	}
	if(!(drawmode & GOURAD)) {
		ColorBlend c;
		for(i=0; i<nsides; i++)
			c += (Color)iv[i]; 
		color = c;
		if(!(drawmode & DITHER))
			pix = cmap->match(color);
	}
	for(i=0; i<nsides; i++) {
		//if(iv[i].x < 0 || iv[i].x > 319 || iv[i].y < 0 || iv[i].y
		//	> 319) {
		//	printf("Whoa! x,y=%d,%d\n", iv[i].x, iv[i].y);
		//	exit(1);
		//}
		int a = i;
		int b = i+1;
		if(b >= nsides)
			b = 0;
		int dx = iv[b].x-iv[a].x;
		if(!dx)
			continue;
		if(dx < 0) {
			a = b; b = i;
			dx = -dx;
		}
		side[ns].y.Slope(iv[a].y, iv[b].y, dx);
		if(drawmode & ZBUF)
			side[ns].z.Slope(iv[a].z, iv[b].z, dx);
		if(drawmode & GOURAD)
			side[ns].c.ColorSlope(iv[a], iv[b], dx);
		side[ns].x1 = iv[a].x;
		side[ns].x2 = iv[b].x;
		if(iv[a].x < xmin)
			xmin = iv[a].x;
		if(iv[b].x > xmax)
			xmax = iv[b].x;
		ns++;
        }
        if(ns < 2)
                return;
        for(int x=xmin; x<=xmax; x++) {
                short y1 = SHRT_MAX;
                short y2 = SHRT_MIN;
		short z1 = 0;
		short z2 = 0;
		Color c1;
		Color c2;
                for(i=0; i<ns; i++)
                    if(x >= side[i].x1 && x <= side[i].x2) {
			if(side[i].y < y1) {
				y1 = side[i].y;
				z1 = side[i].z;
				c1 = side[i].c;
			}
			if(side[i].y > y2) {
				y2 = side[i].y;
				z2 = side[i].z;
				c2 = side[i].c;
			}
			++side[i].y;
			if(drawmode & ZBUF)
				++side[i].z;
			if(drawmode & GOURAD)
				++side[i].c;
		    }
                if(y1 < SHRT_MAX) 
			switch(drawmode) {
			case SOLID:
                        	raw_vline(x, y1, y2, pix); break;
			case DITHER:
                        	raw_vline(x, y1, y2, color); break;
			case GOURAD:
			case GOURAD_DITHER:
                        	raw_vline(x, y1, y2, c1, c2); break;
			case ZBUF:
                        	raw_vline(x, y1, y2, z1, z2, pix); break;
			case ZBUF_DITHER:
                        	raw_vline(x, y1, y2, z1, z2, color); break;
			case ZBUF_GOURAD:
			case ZBUF_GOURAD_DITHER:
                        	raw_vline(x, y1, y2, z1, z2, c1, c2); break;
			default: 	
				;
			}
        }
}


bool ViewPort::read_mouse(mouse_state *mouse)
{
	screen->read_mouse(mouse);
	if(mouse->x >= xp && mouse->x < xp+wid
	   && mouse->y >= yp && mouse->y < yp+hgt) {
		mouse->x = mouse->x-xp;
		mouse->y = mouse->y-yp;
		return true;
	} else
		return false;
}

