// ViewPort_svga.h
//	ShagRend3D viewport specifications

#ifndef __ViewPort_h__
#define __ViewPort_h__


#include <limits.h>
#include "types.h"
#include "Color.h"
#include "Screen.h"
#include "IVertex.h"
#include "ioport.h"
#include "svga.h"


typedef ushort ZPixel;
#define ZBUF_MIN	0
#define ZBUF_MAX	SHRT_MAX


class ViewPort {
	Screen	*screen;
	ColorMap *cmap;
	Pixel	**frame_y;
	Pixel	**frame_yb[4];
	int	fcount;		// total frame count
	ZPixel	*zbuf;
	ZPixel	**zbuf_y;
	int	xp, yp;
	int	wid, hgt;
	int	xctr, yctr;
	bool	zbuffer;
	bool	wireframe;
	int	pitch;
	int	vpitch;
	int	zbuf_size;
	enum DrawMode {
		SOLID = 0,
		DITHER,
		GOURAD,
		GOURAD_DITHER,
		ZBUF,
		ZBUF_DITHER,
		ZBUF_GOURAD,
		ZBUF_GOURAD_DITHER,
		WIREFRAME,
		WIREFRAME_DITHER,
		WIREFRAME_GOURAD,
		WIREFRAME_GOURAD_DITHER,
		WIREFRAME_ZBUF,
		WIREFRAME_ZBUF_DITHER,
		WIREFRAME_ZBUF_GOURAD,
		WIREFRAME_ZBUF_GOURAD_DITHER
	} drawmode;
	void init();
	void raw_put_pixel(register int x, int y, int z, Color &c) {
		ZPixel *zp = &zbuf_y[y][x];
		if(z < *zp) {
			*zp = z;
#ifdef STEREO
			frame[y][x] = cmap->match(c);
#else
			set_mask(1 << (x&3));
			frame_y[y][x/4] = cmap->match(c);
#endif
		}
	}

	void raw_vline(register int x, int y, int y2, int z1, int z2,	
			Color &c1, Color &c2);
	void raw_hline(register int x, int x2, int y, int z1, int z2,
			Color &c1, Color &c2);
	void raw_line(int x1, int y1, int x2, int y2, int z1, int z2,
			Color &c1, Color &c2);
	void raw_vline(register int x, int y, int y2, int z1, int z2,	
			Color &c);
	void raw_hline(register int x, int x2, int y, int z1, int z2,
			Color &c);
	void raw_line(int x1, int y1, int x2, int y2, int z1, int z2,
			Color &c);
	void raw_vline(register int x, int y, int y2, int z1, int z2,	
			Pixel pix);
	void raw_hline(register int x, int x2, int y, int z1, int z2,
			Pixel pix);
	void raw_line(int x1, int y1, int x2, int y2, int z1, int z2,
			Pixel pix);
	void raw_vline(register int x, int y, int y2, Color &c1, Color &c2);
	void raw_hline(register int x, int x2, int y, Color &c1, Color &c2);
	void raw_line(int x1, int y1, int x2, int y2, Color &c1, Color &c2);
	void raw_vline(register int x, int y, int y2, Color &c);
	void raw_hline(register int x, int x2, int y, Color &c);
	void raw_line(int x1, int y1, int x2, int y2, Color &c);
	void raw_vline(register int x, int y, int y2, Pixel pix);
	void raw_hline(register int x, int x2, int y, Pixel pix);
	void raw_line(int x1, int y1, int x2, int y2, Pixel pix);

	void raw_vline_stereo(register int x, int y, int y2, int z1, int z2,	
			Color &c1, Color &c2);
	void raw_hline_stereo(register int x, int x2, int y, int z1, int z2,
			Color &c1, Color &c2);
	void raw_line_stereo(int x1, int y1, int x2, int y2, int z1, int z2,
			Color &c1, Color &c2);
	void raw_vline_stereo(register int x, int y, int y2, int z1, int z2,	
			Color &c);
	void raw_hline_stereo(register int x, int x2, int y, int z1, int z2,
			Color &c);
	void raw_line_stereo(int x1, int y1, int x2, int y2, int z1, int z2,
			Color &c);
	void raw_vline_stereo(register int x, int y, int y2, int z1, int z2,	
			Pixel pix);
	void raw_hline_stereo(register int x, int x2, int y, int z1, int z2,
			Pixel pix);
	void raw_line_stereo(int x1, int y1, int x2, int y2, int z1, int z2,
			Pixel pix);
	void raw_vline_stereo(register int x, int y, int y2, Color &c1, Color &c2);
	void raw_hline_stereo(register int x, int x2, int y, Color &c1, Color &c2);
	void raw_line_stereo(int x1, int y1, int x2, int y2, Color &c1, Color &c2);
	void raw_vline_stereo(register int x, int y, int y2, Color &c);
	void raw_hline_stereo(register int x, int x2, int y, Color &c);
	void raw_line_stereo(int x1, int y1, int x2, int y2, Color &c);
	void raw_vline_stereo(register int x, int y, int y2, Pixel pix);
	void raw_hline_stereo(register int x, int x2, int y, Pixel pix);
	void raw_line_stereo(int x1, int y1, int x2, int y2, Pixel pix);

public:
	ViewPort(Screen &s, int x, int y, int w, int h);
	ViewPort(Screen &s);
	~ViewPort();
	void set_mask(register int mask) { screen->set_mask(mask); }
	void set_dither(bool on)
		{ if(on) drawmode = (DrawMode)(drawmode | DITHER);
		    else drawmode = (DrawMode)(drawmode & ~DITHER); }
	void set_gourad(bool on)
		{ if(on) drawmode = (DrawMode)(drawmode | GOURAD);
		    else drawmode = (DrawMode)(drawmode & ~GOURAD); }
	void set_zbuf(bool on)
		{ if(on) drawmode = (DrawMode)(drawmode | ZBUF);
		    else drawmode = (DrawMode)(drawmode & ~ZBUF); }
	void set_wireframe(bool on)
		{ if(on) drawmode = (DrawMode)(drawmode | WIREFRAME);
		    else drawmode = (DrawMode)(drawmode & ~WIREFRAME); }
	bool dither_set()	{ return drawmode & DITHER; }
	bool gourad_set()	{ return drawmode & GOURAD; }
	bool zbuf_set()		{ return drawmode & ZBUF; }
	bool wireframe_set()	{ return drawmode & WIREFRAME; }
	void clear();
	void update();
	void put_pixel(int x, int y, int z, Color &c) {
		if(x < 0 || x > wid-1 || y < 0 || y > hgt-1)
			return;
		raw_put_pixel(x, y, z, c);
	}
	void hline(int x1, int x2, int y, int z1, int z2,
				Color &c1, Color &c2) {
		raw_hline(x1, x2, y, z1, z2, c1, c2);
	}
	void vline(int x, int y1, int y2, int z1, int z2,
				Color &c1, Color &c2) {
		raw_vline(x, y1, y2, z1, z2, c1, c2);
	}
	void line(int x1, int y1, int x2, int y2, int z1, int z2,
				Color &c1, Color &c2) {
		raw_line(x1, y1, x2, y2, z1, z2, c1, c2);
	}
	void draw_polygon(IVertex iv[], int nsides);
	void fill_polygon(IVertex iv[], int nsides);
	int fps_x1000();
	int fps() { return fps_x1000()/1000; }
	int get_key() { return screen->get_key(); }
	bool read_mouse(mouse_state *state);
	int width()	{ return wid; }
	int height()	{ return hgt; }
	int xcenter()	{ return xctr; }
	int ycenter()	{ return yctr; }
	int framecount() { return fcount; }
};

#ifdef INLINE_RAWLINE
#include "rawline.C"
#endif

#endif
