// linefunc.C
//	generate raw line drawing functions for sr3d


#undef _ZBUF
#ifdef DRAWMODE_ZBUF
#define _ZBUF
#else
#define _ZBUF	/##/
#endif

#undef _GOURAD
#ifdef DRAWMODE_GOURAD
#define _GOURAD
#else
#define _GOURAD	/##/
#endif

#undef _DITHER
#ifdef DRAWMODE_DITHER
#define _DITHER
#else
#define _DITHER /##/
#endif

#undef _DITHER_NG
#if defined(DRAWMODE_DITHER) && !defined(DRAWMODE_GOURAD)
#define _DITHER_NG
#else
#define _DITHER_NG /##/
#endif

#undef _ND_NG
#if !defined(DRAWMODE_DITHER) && !defined(DRAWMODE_GOURAD)
#define _ND_NG
#else
#define _ND_NG /##/
#endif

#ifdef STEREO
#define FUNC(t,n)		__INLINE__ t n##_stereo
#else
#define FUNC(t,n)		__INLINE__ t n
#endif


FUNC(void,ViewPort::raw_vline) (
		register int x, int y1, int y2,
_ZBUF		int z1, int z2,
_GOURAD		Color &c1, Color &c2
_DITHER_NG	Color &c
_ND_NG		Pixel pix
		)
{
register int y = y1;
int dy = y2-y1;
_GOURAD ColorSlope c(c1, c2, dy);
_ZBUF Slope z(z1, z2, dy);
_ZBUF ZPixel *zp = &zbuf_y[y][x];
#ifdef STEREO
	Pixel *pp = &frame_y[y][x];
#else
	set_mask(1 << (x&3));
	Pixel *pp = &frame_y[y][x/4];
#endif
	while(dy-- >= 0) {
_ZBUF		if(z < *zp) {
_ZBUF			*zp = z;
_DITHER			*pp = cmap->match(c);
_ND_NG			*pp = pix;
_ZBUF		}
_ZBUF		++z;
_ZBUF		zp += pitch;
		pp += vpitch;
_GOURAD		++c;
	}
}



#ifdef STEREO

#if defined(DRAWMODE_ZBUF) && defined(DRAWMODE_DITHER) && defined(DRAWMODE_GOURAD)
FUNC(void, ViewPort::raw_hline) (
			register int x, int x2, int y,
			int z1, int z2, Color &c1, Color &c2)
{
register Pixel *pp = &frame_y[y][x/4];
register ZPixel *zp = &zbuf_y[y][x];
int dx = x2-x;
ColorSlope c(c1, c2, dx);
Slope z(z1, z2, dx);
	while(dx-- >= 0) {
		if(z > *zp) {
			*zp = z;
			*pp = cmap->match(c);
		}
		++pp;
		++z;
		++c;
	}
}
#endif

#else

#if defined(DRAWMODE_ZBUF) && defined(DRAWMODE_DITHER) && defined(DRAWMODE_GOURAD)
FUNC(void, ViewPort::raw_hline) (
			register int x, int x2, int y,
			int z, int z2, Color &c, Color &c2)
{
register Pixel *pp = &frame_y[y][x/4];
register ZPixel *zp = &zbuf_y[y][x];
register int max = 0;	
register int pix = 0, oldpix = 0;
Pixel *savepp = 0;
	while(x <= x2) {
		pix = cmap->match(c);
		if(max && (z >= *zp || pix != oldpix)) {
			if(max >= 4) {
				set_mask(0xf);
				asm("movb %%al,%%ah; movw %%ax,%%dx;"
				    "shll $16,%%eax; movw %%dx,%%ax;"
				    "shrl $2,%%ecx; movl %%ecx,%%edx;"
				    "shrl $2,%%ecx; jz 1f;"
				    "rep; stosl;"
				    "1: movl %%edx,%%ecx;"
				    "andl $3,%%ecx; jz 2f;"
				    "rep; stosb;"
				    "2:"  :"=D"(savepp) :
					"a"(oldpix), "c"(max),
					"D"(savepp) :
					"eax", "ecx", "edx");
				max &= 3;
			}
			while(max) {
				set_mask(1<<((x-max)&3));
				*savepp = oldpix;
				max--;
			}
		}
		if(z < *zp) {
			*zp = z;
			if(max)
				max++;
			else if(!(x&0xf)) {
				savepp = pp;
				oldpix = pix;
				max++;
			} else {
				set_mask(1 << (x&3));
				*pp = pix;
			}
		} 
		zp++;
		if(!(++x&3))
			pp++;
	}
	if(max) {
		if(max >= 4) {
			set_mask(0xf);
			asm("movb %%al,%%ah; movw %%ax,%%dx;"
			    "shll $16,%%eax; movw %%dx,%%ax;"
			    "shrl $2,%%ecx; movl %%ecx,%%edx;"
			    "shrl $2,%%ecx; jz 1f;"
			    "rep; stosl;"
			    "1: movl %%edx,%%ecx;"
			    "andl $3,%%ecx; jz 2f;"
			    "rep; stosb;"
			    "2:"  : "=D"(savepp) :
				"a"(oldpix), "c"(max),
				"D"(savepp) :
				"eax", "ecx", "edx");
			max &= 3;
		}
		while(max) {
			set_mask(1<<((x-max)&3));
			*savepp = oldpix;
			max--;
		}
	}
}
#endif

#endif	// STEREO



FUNC(void, ViewPort::raw_line) (
		int x1, int y1,
		int x2, int y2,
_ZBUF		int z1, int z2,
_GOURAD		Color &c1, Color &c2
_DITHER_NG	Color &c
_ND_NG		Pixel pix
		)
{
int dy = y2-y1;
int ady = abs(dy);
int dx = x2-x1;
int adx = abs(dx);
	if(adx > ady) {
		int x = x1;
		Slope y(y1, y2, adx);
_ZBUF		Slope z(z1, z2, adx);
_GOURAD		ColorSlope c(c1, c2, adx);
		int xinc = sign(dx);
		do {
_ZBUF			ZPixel *zp = &zbuf_y[y][x];
_ZBUF			if(z < *zp) {
_ZBUF				*zp = z;
#ifdef STEREO
				frame_y[y][x] 
#else
				set_mask(1 << (x&3));
				frame_y[y][x/4]
#endif
_DITHER					= cmap->match(c);
_ND_NG					= pix;
_ZBUF			}
_ZBUF			++z;
			x += xinc; ++y;
_GOURAD			++c;
		} while(x != x2);
	} else if(ady) {
		Slope x(x1, x2, ady);
		int y = y1;
_ZBUF		Slope z(z1, z2, ady);
_GOURAD		ColorSlope c(c1, c2, ady);
		int yinc = sign(dy);
		do {
_ZBUF			ZPixel *zp = &zbuf_y[y][x];
_ZBUF			if(z < *zp) {
_ZBUF				*zp = z;
#ifdef STEREO
				frame_y[y][x]
#else
				set_mask(1 << (x&3));
				frame_y[y][x/4]
#endif
_DITHER					= cmap->match(c);
_ND_NG					= pix;
_ZBUF			}
_ZBUF			++z;
			++x; y += yinc;
_GOURAD			++c;
		} while(y != y2);
	} else {
_ZBUF		ZPixel *zp = &zbuf_y[y1][x1];
_ZBUF		if(z1 < *zp || z2 < *zp) {
_ZBUF			if(z1 < z2)
_ZBUF				*zp = z1;	
_ZBUF			else
_ZBUF				*zp = z2;
			set_mask(1 << (x1&3));
_GOURAD			frame_y[y1][x1/4] = cmap->match(c1);
_DITHER_NG		frame_y[y1][x1/4] = cmap->match(c);
_ND_NG			frame_y[y1][x1/4] = pix;
_ZBUF		}
	}	
}

