 
 


 
 





 
 






typedef unsigned short ushort;
typedef unsigned long ulong;
typedef unsigned int uint;

typedef unsigned char uchar;




 
 






 
 




 





 
 






class Slope {
	int	y;
	int	d;
	int	incr[2];
	int	yinc[2];
public:
	Slope() { }
	Slope(int y1, int y2, int dx) {
		int dy;
		y = y1;
		yinc[0] = 0;
		if(dx <= 0)
			return;
		if(y2>y1) {
			dy = y2-y1;
			if(dy>dx) {
				yinc[0] = dy/dx;
				dy -= yinc[0]*dx;
			}
			yinc[1] = yinc[0]+1;
		} else {
			dy = y1-y2;
			if(dy>dx) {
				yinc[0] = dy/dx;
				dy -= yinc[0]*dx;
				yinc[0] = -yinc[0];
			}
			yinc[1] = yinc[0]-1;
		}
		incr[0] = dy+dy;
		d = incr[0]-dx;
		incr[1] = d-dx;
	}
	int operator++() {
		y += yinc[d >= 0];
		d += incr[d >= 0];
		return y;
	}
	int operator++(int) {
		int oldy = y;
		++(*this);
		return oldy;
	}
	operator int()	{ return y; }
};

 

















































typedef Slope uSlope;






class Color {
	friend class ColorSlope;
public:
	Color()				{ red=0; green=0; blue=0; }
	Color(int v)			{ red=v; green=v; blue=v; }
	Color(int r, int g, int b)	{ red=r; green=g; blue=b; }
	ushort	red;
	ushort	green;
	ushort	blue;
	Color& operator*= (int f)
		{ red=red*f/65535; green=green*f/65535; blue=blue*f/65535;
		  return *this; }
	Color operator* (int f) { Color c;
		  c.red=red*f/65535; c.green=green*f/65535; c.blue=blue*f/65535;
		  return c; }
};


class ColorBlend {
public:
	int	red;
	int	green;
	int	blue;
	int	n;
	ColorBlend()	{ red=0; green=0; blue=0; n=0; }
	ColorBlend& operator+= (Color &c)
			{ red+=c.red; green+=c.green; blue+=c.blue; n++; }
	operator Color() {
		if(n) return Color(red/n, green/n, blue/n);
		else return Color(0, 0, 0); }
};


class ColorSlope : public Color {
	uint	red_val;
	uint	green_val;
	uint	blue_val;
	int	red_inc;
	int	green_inc;
	int	blue_inc;
public:
	ColorSlope() {}
	ColorSlope(Color &c1, Color &c2, int x) {
		if(x <= 0)
			x = 1;
		red = c1.red;
		red_val = red<<16;
		if(c2.red >= c1.red)
			red_inc = ((c2.red<<16)-red_val)/x;
		else
			red_inc = -((red_val-(c2.red<<16))/x);
		green = c1.green;
		green_val = green<<16;
		if(c2.green >= c1.green)
			green_inc = ((c2.green<<16)-green_val)/x;
		else
			green_inc = -((green_val-(c2.green<<16))/x);
		blue = c1.blue;
		blue_val = blue<<16;
		if(c2.blue >= c1.blue)
			blue_inc = ((c2.blue<<16)-blue_val)/x;
		else
			blue_inc = -((blue_val-(c2.blue<<16))/x);
	}
	ColorSlope& operator++() {
		red_val += red_inc;
		red = red_val >> 16;
		green_val += green_inc;
		green = green_val >> 16;
		blue_val += blue_inc;
		blue = blue_val >> 16;
		return *this;
	}
};


class ColorMap {
	int	*pix;
	Color	*col;
	int	ncolors;
	int	nred;
	int	ngreen;
	int	nblue;
	int	ired[256];
	int	igreen[256];
	int	iblue[256];
	int	pred[256];
	int	pgreen[256];
	int	pblue[256];
	int	seed;
public:
	ColorMap(int size);
	~ColorMap();
	int	size()		{ return ncolors; }
	Color&	color(int i)	{ return col[i]; }
	int&	pixel(int i)	{ return pix[i]; }
	int	match(Color &c) {
			seed = seed*1103515245+12345;
			int qred = ired[c.red>>8]+
			 	((seed&0xff)<pred[c.red>>8]);
			seed = seed*1103515245+12345;
			int qgreen = igreen[c.green>>8]+
    				((seed&0xff)<pgreen[c.green>>8]);
			seed = seed*1103515245+12345;
			int qblue = iblue[c.blue>>8]+
    				((seed&0xff)<pblue[c.blue>>8]);
			 
			 
			 
			return pix[(qred*ngreen+qgreen)*nblue+qblue];
	}
};



 





 
 







static inline void outb(register int port, register int value)
{
	__asm__ volatile ("outb %0,%1"
		: : "a" ((uchar)value), "d" ((ushort)port));
}

static inline void outw(register int port, register int value) {
	__asm__ volatile("outw %0,%1"
		: : "a" ((ushort)value), "d" ((ushort)port));
}

static inline int inb(register int port)
{
	register uchar value;
	__asm__ volatile ("inb %1,%0" : "=a" (value) : "d" ((ushort)port));
	return value;
}

static inline int inw(register int port)
{
	register ushort value;
	__asm__ volatile ("inw %1,%0"
		: "=a" (value)
		: "d" ((ushort)port));
	return value;
}




 





 








 















typedef unsigned char Pixel;


struct mouse_state {
	int	oldx, oldy;
	int	x, y;
	bool	button[3];
	bool	oldbutton[3];
};


class Screen {
	int	wid, hgt;
	ColorMap *cmap;
	Pixel	*vidmem;
	uchar	*font;
	int	vpitch;
	int	framesize;
	static int page;
	static int planemask;
	static int vbase;			 
	static int vofs;			 
public:
	Screen();
	~Screen();
	static int pagebase[4];
	int width()	{ return wid; }
	int height()	{ return hgt; }
	ColorMap* colormap() { return cmap; }
	Pixel* video_base()  { return vidmem; }
	int get_key();
	bool read_mouse(mouse_state *state);
	void write(int page, int x, int y, char *s, int c0, int cinc);
	void write(int page, int x, int y, char *s)
		{ write(page, x, y, s, 251, 0); }
	void write(int x, int y, char *s, int c0, int cinc)
		{ write(page, x, y, s, c0, cinc); }
	void write(int x, int y, char *s)
		{ write(page, x, y, s, 251, 0); }
	void write_all(int x, int y, char *s, int c0, int cinc) {
		write(0, x, y, s, c0, cinc);
		write(1, x, y, s, c0, cinc);
		write(2, x, y, s, c0, cinc); }
	void write_all(int x, int y, char *s)
		{ write(x, y, s, 251, 0); }
	static void set_mask(register int mask) {
		if(mask != planemask) {
			planemask = mask;
			outb(0x3C4 , 0x02);
			outb(0x3C5 , mask);
		}
	}
	static void set_vofs(int adr) {
		inb(0x3DA );
		outw(0x3D4 , 0xd + ((adr & 0xff) << 8));
		outw(0x3D4 , 0xc + ((adr & 0xff00)));
	}
	static int flip_page() {
		set_vofs((vofs = pagebase[page]));
		if(++page > 2)  page = 0;
		vbase = pagebase[page];
		return page;
	}
};



 
 





 


 





 
 




 

















 






 





















 




































 









 





 



 



 






 


































 
 






















 




























 


 



   


















 







 

typedef long double __long_double_t;



 




 





 












 
 

















 








 

 


 


 


 


 



 


 


 


 


 


 


 


 



 




















 













 









 






















 


 


 


 


 



 



 


 




 




































 





  






 










 



 



 




 





 



 







 





 



 








 



 









 




 








 






 




 
 




 
 




 
 



 




 
 




 






 
 






typedef int ivec_t;


class IVector {
public:
	ivec_t x;
	ivec_t y;
	ivec_t z;
	IVector()			{}
	IVector(ivec_t n)       	{ x = n; y = n; z = n; }
	IVector& operator=(ivec_t n) 	{ x = n; y = n; z = n; return *this; }
	IVector(IVector &v)		{ x=v.x; y=v.y; z=v.z; }  
	IVector& operator=(IVector &v)	{ x=v.x; y=v.y; z=v.z; return *this; } 
	IVector(ivec_t nx, ivec_t ny, ivec_t nz)
					{ x = nx; y = ny; z = nz; }
	operator ivec_t ()		{ return x*x+y*y+z*z; }
	IVector& operator*=(ivec_t n)	{ x*=n; y*=n; z*=n; return *this; }
	ivec_t operator*(IVector &v)	{ return x*v.x+y*v.y+z*v.z; }
	IVector& operator+=(IVector &v)
		{ x+=v.x; y+=v.y; z+=v.z; return *this; }
	IVector& operator-=(IVector &v)
		{ x-=v.y; y-=v.y; z-=v.z; return *this; }
	IVector operator+(IVector &b) 
		{ IVector r; r.x=x+b.y; r.y=y+b.y; r.z=z+b.z; return r; }
	IVector operator-(IVector &b) 
		{ IVector r; r.x=x-b.y; r.y=y-b.y; r.z=z-b.z; return r; }
	IVector operator-()
		{ IVector r; r.x=-x; r.y=-y; r.z=-z; return r; }
	IVector& operator^=(IVector &b) {
		IVector a = *this;
		x = a.y*b.z-b.y*a.z;
		y = a.z*b.x-b.z*a.x;
		z = a.x*b.y-b.x*a.y;
		return *this; }
	IVector operator^(IVector &b) {
		IVector r;
		r.x = y*b.z-b.y*z;
		r.y = z*b.x-b.z*x;
		r.z = x*b.y-b.x*y;
		return r; }
	IVector& normalize() {
		ivec_t n = 1/ *this;
		x *= n;
		y *= n;
		z *= n;
		return *this; }
	IVector(IVector &v1, IVector &v2, IVector &v3)
		{ *this = ((v1-v2)^(v1-v3)).normalize(); }
	IVector& rotate_x(ivec_t s, ivec_t c) {
		IVector v;
		v.y = y*c - z*s;
		v.z = z*c + y*s;
		v.x = x;
		*this = v;
		return *this; }
	IVector& rotate_y(ivec_t s, ivec_t c) {
		IVector v;
		v.z = z*c - x*s;
		v.x = x*c + z*s;
		v.y = y;
		*this = v;
		return *this;
	}
	IVector& rotate_z(ivec_t s, ivec_t c) {
		IVector v;
		v.x = x*c - y*s;
		v.y = y*c + x*s;
		v.z = z;
		*this = v;
		return *this;
	}
};





 
 








class IVertex : public IVector, public Color {
public:
	IVertex() {}
	IVertex(IVector &v)		{ *(IVector*)this = v; }
	IVertex& operator=(IVector &v)	{ *(IVector*)this = v; return *this; }
	IVertex(Color &c)		{ *(Color*)this = c; }
	IVertex& operator=(Color &c)	{ *(Color*)this = c; return *this; }
	IVertex(ivec_t x, ivec_t y, ivec_t z,
		ushort red, ushort green, ushort blue)
			: IVector(x, y, z), Color(red, green, blue) {}
};




 



 





typedef ushort ZPixel;




class ViewPort {
	Screen	*screen;
	ColorMap *cmap;
	Pixel	**frame_y;
	Pixel	**frame_yb[4];
	int	fcount;		 
	ZPixel	*zbuf;
	ZPixel	**zbuf_y;
	int	xp, yp;
	int	wid, hgt;
	int	xctr, yctr;
	bool	zbuffer;
	bool	wireframe;
	int	pitch;
	int	vpitch;
	int	zbuf_size;
	enum DrawMode {
		SOLID = 0,
		DITHER,
		GOURAD,
		GOURAD_DITHER,
		ZBUF,
		ZBUF_DITHER,
		ZBUF_GOURAD,
		ZBUF_GOURAD_DITHER,
		WIREFRAME,
		WIREFRAME_DITHER,
		WIREFRAME_GOURAD,
		WIREFRAME_GOURAD_DITHER,
		WIREFRAME_ZBUF,
		WIREFRAME_ZBUF_DITHER,
		WIREFRAME_ZBUF_GOURAD,
		WIREFRAME_ZBUF_GOURAD_DITHER
	} drawmode;
	void init();
	void raw_put_pixel(register int x, int y, int z, Color &c) {
		ZPixel *zp = &zbuf_y[y][x];
		if(z < *zp) {
			*zp = z;

			set_mask(1 << (x&3));
			frame_y[y][x/4] = cmap->match(c);

		}
	}
	void raw_vline(register int x, int y, int y2, int z1, int z2,	
			Color &c1, Color &c2);
	void raw_hline(register int x, int x2, int y, int z1, int z2,
			Color &c1, Color &c2);
	void raw_line(int x1, int y1, int x2, int y2, int z1, int z2,
			Color &c1, Color &c2);
	void raw_vline(register int x, int y, int y2, int z1, int z2,	
			Color &c);
	void raw_hline(register int x, int x2, int y, int z1, int z2,
			Color &c);
	void raw_line(int x1, int y1, int x2, int y2, int z1, int z2,
			Color &c);
	void raw_vline(register int x, int y, int y2, int z1, int z2,	
			Pixel pix);
	void raw_hline(register int x, int x2, int y, int z1, int z2,
			Pixel pix);
	void raw_line(int x1, int y1, int x2, int y2, int z1, int z2,
			Pixel pix);
	void raw_vline(register int x, int y, int y2, Color &c1, Color &c2);
	void raw_hline(register int x, int x2, int y, Color &c1, Color &c2);
	void raw_line(int x1, int y1, int x2, int y2, Color &c1, Color &c2);
	void raw_vline(register int x, int y, int y2, Color &c);
	void raw_hline(register int x, int x2, int y, Color &c);
	void raw_line(int x1, int y1, int x2, int y2, Color &c);
	void raw_vline(register int x, int y, int y2, Pixel pix);
	void raw_hline(register int x, int x2, int y, Pixel pix);
	void raw_line(int x1, int y1, int x2, int y2, Pixel pix);
public:
	ViewPort(Screen &s, int x, int y, int w, int h);
	ViewPort(Screen &s);
	~ViewPort();
	void set_mask(register int mask) { screen->set_mask(mask); }
	void set_dither(bool on)
		{ if(on) drawmode = (DrawMode)(drawmode | DITHER);
		    else drawmode = (DrawMode)(drawmode & ~DITHER); }
	void set_gourad(bool on)
		{ if(on) drawmode = (DrawMode)(drawmode | GOURAD);
		    else drawmode = (DrawMode)(drawmode & ~GOURAD); }
	void set_zbuf(bool on)
		{ if(on) drawmode = (DrawMode)(drawmode | ZBUF);
		    else drawmode = (DrawMode)(drawmode & ~ZBUF); }
	void set_wireframe(bool on)
		{ if(on) drawmode = (DrawMode)(drawmode | WIREFRAME);
		    else drawmode = (DrawMode)(drawmode & ~WIREFRAME); }
	bool dither_set()	{ return drawmode & DITHER; }
	bool gourad_set()	{ return drawmode & GOURAD; }
	bool zbuf_set()		{ return drawmode & ZBUF; }
	bool wireframe_set()	{ return drawmode & WIREFRAME; }
	void clear();
	void update();
	void put_pixel(int x, int y, int z, Color &c) {
		if(x < 0 || x > wid-1 || y < 0 || y > hgt-1)
			return;
		raw_put_pixel(x, y, z, c);
	}
	void hline(int x1, int x2, int y, int z1, int z2,
				Color &c1, Color &c2) {
		raw_hline(x1, x2, y, z1, z2, c1, c2);
	}
	void vline(int x, int y1, int y2, int z1, int z2,
				Color &c1, Color &c2) {
		raw_vline(x, y1, y2, z1, z2, c1, c2);
	}
	void line(int x1, int y1, int x2, int y2, int z1, int z2,
				Color &c1, Color &c2) {
		raw_line(x1, y1, x2, y2, z1, z2, c1, c2);
	}
	void draw_polygon(IVertex iv[], int nsides);
	void fill_polygon(IVertex iv[], int nsides);
	int fps_x1000();
	int fps() { return fps_x1000()/1000; }
	int get_key() { return screen->get_key(); }
	bool read_mouse(mouse_state *state);
	int width()	{ return wid; }
	int height()	{ return hgt; }
	int xcenter()	{ return xctr; }
	int ycenter()	{ return yctr; }
	int framecount() { return fcount; }
};





 


















 
 





























  void ViewPort::raw_vline(
		register int x, int y1, int y2,
 		int z1, int z2,
 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
register int y = y1;
int dy = y2-y1;
  ColorSlope c(c1, c2, dy);
  Slope z(z1, z2, dy);
  ZPixel *zp = &zbuf_y[y][x];
ECHO(#ifdef STEREO)
	Pixel *pp = &frame_y[y][x];
ECHO(#else)
	set_mask(1 << (x&3));
	Pixel *pp = &frame_y[y][x/4];
ECHO(#endif)
	while(dy-- >= 0) {
 		if(z < *zp) {
 			*zp = z;
 			*pp = cmap->match(c);
 			*pp = pix;
 		}
 		++z;
 		zp += pitch;
		pp += vpitch;
 		++c;
	}
}






  void ViewPort::raw_hline(register int x, int x2, int y,
			int z, int z2, Color &c, Color &c2)
{
register Pixel *pp = &frame_y[y][x/4];
register ZPixel *zp = &zbuf_y[y][x];
register int max = 0;	
register int pix = 0, oldpix = 0;
Pixel *savepp = 0;
	while(x <= x2) {
		pix = cmap->match(c);
		if(max && (z >= *zp || pix != oldpix)) {
			if(max >= 4) {
				set_mask(0xf);
				asm("movb %%al,%%ah; movw %%ax,%%dx;"
				    "shll $16,%%eax; movw %%dx,%%ax;"
				    "shrl $2,%%ecx; movl %%ecx,%%edx;"
				    "shrl $2,%%ecx; jz 1f;"
				    "rep; stosl;"
				    "1: movl %%edx,%%ecx;"
				    "andl $3,%%ecx; jz 2f;"
				    "rep; stosb;"
				    "2:"  :"=D"(savepp) :
					"a"(oldpix), "c"(max),
					"D"(savepp) :
					"eax", "ecx", "edx");
				max &= 3;
			}
			while(max) {
				set_mask(1<<((x-max)&3));
				*savepp = oldpix;
				max--;
			}
		}
		if(z < *zp) {
			*zp = z;
			if(max)
				max++;
			else if(!(x&0xf)) {
				savepp = pp;
				oldpix = pix;
				max++;
			} else {
				set_mask(1 << (x&3));
				*pp = pix;
			}
		} 
		zp++;
		if(!(++x&3))
			pp++;
	}
	if(max) {
		if(max >= 4) {
			set_mask(0xf);
			asm("movb %%al,%%ah; movw %%ax,%%dx;"
			    "shll $16,%%eax; movw %%dx,%%ax;"
			    "shrl $2,%%ecx; movl %%ecx,%%edx;"
			    "shrl $2,%%ecx; jz 1f;"
			    "rep; stosl;"
			    "1: movl %%edx,%%ecx;"
			    "andl $3,%%ecx; jz 2f;"
			    "rep; stosb;"
			    "2:"  : "=D"(savepp) :
				"a"(oldpix), "c"(max),
				"D"(savepp) :
				"eax", "ecx", "edx");
			max &= 3;
		}
		while(max) {
			set_mask(1<<((x-max)&3));
			*savepp = oldpix;
			max--;
		}
	}
}






  void ViewPort::raw_line(
		int x1, int y1,
		int x2, int y2,
 		int z1, int z2,
 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
int dy = y2-y1;
int ady = ((( dy ) >= 0) ? ( dy ) : -( dy )) ;
int dx = x2-x1;
int adx = ((( dx ) >= 0) ? ( dx ) : -( dx )) ;
	if(adx > ady) {
		int x = x1;
		Slope y(y1, y2, adx);
 		Slope z(z1, z2, adx);
 		ColorSlope c(c1, c2, adx);
		int xinc = ((( dx ) > 0) ? 1 : -1 ) ;
		do {
 			ZPixel *zp = &zbuf_y[y][x];
 			if(z < *zp) {
 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x] 
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
 					= cmap->match(c);
 					= pix;
 			}
 			++z;
			x += xinc; ++y;
 			++c;
		} while(x != x2);
	} else if(ady) {
		Slope x(x1, x2, ady);
		int y = y1;
 		Slope z(z1, z2, ady);
 		ColorSlope c(c1, c2, ady);
		int yinc = ((( dy ) > 0) ? 1 : -1 ) ;
		do {
 			ZPixel *zp = &zbuf_y[y][x];
 			if(z < *zp) {
 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x]
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
 					= cmap->match(c);
 					= pix;
 			}
 			++z;
			++x; y += yinc;
 			++c;
		} while(y != y2);
	} else {
 		ZPixel *zp = &zbuf_y[y1][x1];
 		if(z1 < *zp || z2 < *zp) {
 			if(z1 < z2)
 				*zp = z1;	
 			else
 				*zp = z2;
			set_mask(1 << (x1&3));
 			frame_y[y1][x1/4] = cmap->match(c1);
// 		frame_y[y1][x1/4] = cmap->match(c);
 			frame_y[y1][x1/4] = pix;
 		}
	}	
}



 
 





























  void ViewPort::raw_vline(
		register int x, int y1, int y2,
// 		int z1, int z2,
 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
register int y = y1;
int dy = y2-y1;
  ColorSlope c(c1, c2, dy);
//  Slope z(z1, z2, dy);
//  ZPixel *zp = &zbuf_y[y][x];
ECHO(#ifdef STEREO)
	Pixel *pp = &frame_y[y][x];
ECHO(#else)
	set_mask(1 << (x&3));
	Pixel *pp = &frame_y[y][x/4];
ECHO(#endif)
	while(dy-- >= 0) {
// 		if(z < *zp) {
// 			*zp = z;
 			*pp = cmap->match(c);
 			*pp = pix;
// 		}
// 		++z;
// 		zp += pitch;
		pp += vpitch;
 		++c;
	}
}











  void ViewPort::raw_line(
		int x1, int y1,
		int x2, int y2,
// 		int z1, int z2,
 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
int dy = y2-y1;
int ady = ((( dy ) >= 0) ? ( dy ) : -( dy )) ;
int dx = x2-x1;
int adx = ((( dx ) >= 0) ? ( dx ) : -( dx )) ;
	if(adx > ady) {
		int x = x1;
		Slope y(y1, y2, adx);
// 		Slope z(z1, z2, adx);
 		ColorSlope c(c1, c2, adx);
		int xinc = ((( dx ) > 0) ? 1 : -1 ) ;
		do {
// 			ZPixel *zp = &zbuf_y[y][x];
// 			if(z < *zp) {
// 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x] 
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
 					= cmap->match(c);
 					= pix;
// 			}
// 			++z;
			x += xinc; ++y;
 			++c;
		} while(x != x2);
	} else if(ady) {
		Slope x(x1, x2, ady);
		int y = y1;
// 		Slope z(z1, z2, ady);
 		ColorSlope c(c1, c2, ady);
		int yinc = ((( dy ) > 0) ? 1 : -1 ) ;
		do {
// 			ZPixel *zp = &zbuf_y[y][x];
// 			if(z < *zp) {
// 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x]
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
 					= cmap->match(c);
 					= pix;
// 			}
// 			++z;
			++x; y += yinc;
 			++c;
		} while(y != y2);
	} else {
// 		ZPixel *zp = &zbuf_y[y1][x1];
// 		if(z1 < *zp || z2 < *zp) {
// 			if(z1 < z2)
// 				*zp = z1;	
// 			else
// 				*zp = z2;
			set_mask(1 << (x1&3));
 			frame_y[y1][x1/4] = cmap->match(c1);
// 		frame_y[y1][x1/4] = cmap->match(c);
 			frame_y[y1][x1/4] = pix;
// 		}
	}	
}



 
 





























  void ViewPort::raw_vline(
		register int x, int y1, int y2,
// 		int z1, int z2,
// 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
register int y = y1;
int dy = y2-y1;
//  ColorSlope c(c1, c2, dy);
//  Slope z(z1, z2, dy);
//  ZPixel *zp = &zbuf_y[y][x];
ECHO(#ifdef STEREO)
	Pixel *pp = &frame_y[y][x];
ECHO(#else)
	set_mask(1 << (x&3));
	Pixel *pp = &frame_y[y][x/4];
ECHO(#endif)
	while(dy-- >= 0) {
// 		if(z < *zp) {
// 			*zp = z;
 			*pp = cmap->match(c);
 			*pp = pix;
// 		}
// 		++z;
// 		zp += pitch;
		pp += vpitch;
// 		++c;
	}
}











  void ViewPort::raw_line(
		int x1, int y1,
		int x2, int y2,
// 		int z1, int z2,
// 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
int dy = y2-y1;
int ady = ((( dy ) >= 0) ? ( dy ) : -( dy )) ;
int dx = x2-x1;
int adx = ((( dx ) >= 0) ? ( dx ) : -( dx )) ;
	if(adx > ady) {
		int x = x1;
		Slope y(y1, y2, adx);
// 		Slope z(z1, z2, adx);
// 		ColorSlope c(c1, c2, adx);
		int xinc = ((( dx ) > 0) ? 1 : -1 ) ;
		do {
// 			ZPixel *zp = &zbuf_y[y][x];
// 			if(z < *zp) {
// 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x] 
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
 					= cmap->match(c);
 					= pix;
// 			}
// 			++z;
			x += xinc; ++y;
// 			++c;
		} while(x != x2);
	} else if(ady) {
		Slope x(x1, x2, ady);
		int y = y1;
// 		Slope z(z1, z2, ady);
// 		ColorSlope c(c1, c2, ady);
		int yinc = ((( dy ) > 0) ? 1 : -1 ) ;
		do {
// 			ZPixel *zp = &zbuf_y[y][x];
// 			if(z < *zp) {
// 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x]
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
 					= cmap->match(c);
 					= pix;
// 			}
// 			++z;
			++x; y += yinc;
// 			++c;
		} while(y != y2);
	} else {
// 		ZPixel *zp = &zbuf_y[y1][x1];
// 		if(z1 < *zp || z2 < *zp) {
// 			if(z1 < z2)
// 				*zp = z1;	
// 			else
// 				*zp = z2;
			set_mask(1 << (x1&3));
// 			frame_y[y1][x1/4] = cmap->match(c1);
// 		frame_y[y1][x1/4] = cmap->match(c);
 			frame_y[y1][x1/4] = pix;
// 		}
	}	
}



 
 





























  void ViewPort::raw_vline(
		register int x, int y1, int y2,
 		int z1, int z2,
// 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
register int y = y1;
int dy = y2-y1;
//  ColorSlope c(c1, c2, dy);
  Slope z(z1, z2, dy);
  ZPixel *zp = &zbuf_y[y][x];
ECHO(#ifdef STEREO)
	Pixel *pp = &frame_y[y][x];
ECHO(#else)
	set_mask(1 << (x&3));
	Pixel *pp = &frame_y[y][x/4];
ECHO(#endif)
	while(dy-- >= 0) {
 		if(z < *zp) {
 			*zp = z;
 			*pp = cmap->match(c);
 			*pp = pix;
 		}
 		++z;
 		zp += pitch;
		pp += vpitch;
// 		++c;
	}
}











  void ViewPort::raw_line(
		int x1, int y1,
		int x2, int y2,
 		int z1, int z2,
// 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
int dy = y2-y1;
int ady = ((( dy ) >= 0) ? ( dy ) : -( dy )) ;
int dx = x2-x1;
int adx = ((( dx ) >= 0) ? ( dx ) : -( dx )) ;
	if(adx > ady) {
		int x = x1;
		Slope y(y1, y2, adx);
 		Slope z(z1, z2, adx);
// 		ColorSlope c(c1, c2, adx);
		int xinc = ((( dx ) > 0) ? 1 : -1 ) ;
		do {
 			ZPixel *zp = &zbuf_y[y][x];
 			if(z < *zp) {
 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x] 
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
 					= cmap->match(c);
 					= pix;
 			}
 			++z;
			x += xinc; ++y;
// 			++c;
		} while(x != x2);
	} else if(ady) {
		Slope x(x1, x2, ady);
		int y = y1;
 		Slope z(z1, z2, ady);
// 		ColorSlope c(c1, c2, ady);
		int yinc = ((( dy ) > 0) ? 1 : -1 ) ;
		do {
 			ZPixel *zp = &zbuf_y[y][x];
 			if(z < *zp) {
 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x]
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
 					= cmap->match(c);
 					= pix;
 			}
 			++z;
			++x; y += yinc;
// 			++c;
		} while(y != y2);
	} else {
 		ZPixel *zp = &zbuf_y[y1][x1];
 		if(z1 < *zp || z2 < *zp) {
 			if(z1 < z2)
 				*zp = z1;	
 			else
 				*zp = z2;
			set_mask(1 << (x1&3));
// 			frame_y[y1][x1/4] = cmap->match(c1);
// 		frame_y[y1][x1/4] = cmap->match(c);
 			frame_y[y1][x1/4] = pix;
 		}
	}	
}



 
 





























  void ViewPort::raw_vline(
		register int x, int y1, int y2,
 		int z1, int z2,
// 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
register int y = y1;
int dy = y2-y1;
//  ColorSlope c(c1, c2, dy);
  Slope z(z1, z2, dy);
  ZPixel *zp = &zbuf_y[y][x];
ECHO(#ifdef STEREO)
	Pixel *pp = &frame_y[y][x];
ECHO(#else)
	set_mask(1 << (x&3));
	Pixel *pp = &frame_y[y][x/4];
ECHO(#endif)
	while(dy-- >= 0) {
 		if(z < *zp) {
 			*zp = z;
// 			*pp = cmap->match(c);
 			*pp = pix;
 		}
 		++z;
 		zp += pitch;
		pp += vpitch;
// 		++c;
	}
}











  void ViewPort::raw_line(
		int x1, int y1,
		int x2, int y2,
 		int z1, int z2,
// 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
int dy = y2-y1;
int ady = ((( dy ) >= 0) ? ( dy ) : -( dy )) ;
int dx = x2-x1;
int adx = ((( dx ) >= 0) ? ( dx ) : -( dx )) ;
	if(adx > ady) {
		int x = x1;
		Slope y(y1, y2, adx);
 		Slope z(z1, z2, adx);
// 		ColorSlope c(c1, c2, adx);
		int xinc = ((( dx ) > 0) ? 1 : -1 ) ;
		do {
 			ZPixel *zp = &zbuf_y[y][x];
 			if(z < *zp) {
 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x] 
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
// 					= cmap->match(c);
 					= pix;
 			}
 			++z;
			x += xinc; ++y;
// 			++c;
		} while(x != x2);
	} else if(ady) {
		Slope x(x1, x2, ady);
		int y = y1;
 		Slope z(z1, z2, ady);
// 		ColorSlope c(c1, c2, ady);
		int yinc = ((( dy ) > 0) ? 1 : -1 ) ;
		do {
 			ZPixel *zp = &zbuf_y[y][x];
 			if(z < *zp) {
 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x]
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
// 					= cmap->match(c);
 					= pix;
 			}
 			++z;
			++x; y += yinc;
// 			++c;
		} while(y != y2);
	} else {
 		ZPixel *zp = &zbuf_y[y1][x1];
 		if(z1 < *zp || z2 < *zp) {
 			if(z1 < z2)
 				*zp = z1;	
 			else
 				*zp = z2;
			set_mask(1 << (x1&3));
// 			frame_y[y1][x1/4] = cmap->match(c1);
// 		frame_y[y1][x1/4] = cmap->match(c);
 			frame_y[y1][x1/4] = pix;
 		}
	}	
}



 
 





























  void ViewPort::raw_vline(
		register int x, int y1, int y2,
// 		int z1, int z2,
// 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
register int y = y1;
int dy = y2-y1;
//  ColorSlope c(c1, c2, dy);
//  Slope z(z1, z2, dy);
//  ZPixel *zp = &zbuf_y[y][x];
ECHO(#ifdef STEREO)
	Pixel *pp = &frame_y[y][x];
ECHO(#else)
	set_mask(1 << (x&3));
	Pixel *pp = &frame_y[y][x/4];
ECHO(#endif)
	while(dy-- >= 0) {
// 		if(z < *zp) {
// 			*zp = z;
// 			*pp = cmap->match(c);
 			*pp = pix;
// 		}
// 		++z;
// 		zp += pitch;
		pp += vpitch;
// 		++c;
	}
}











  void ViewPort::raw_line(
		int x1, int y1,
		int x2, int y2,
// 		int z1, int z2,
// 		Color &c1, Color &c2
// 	Color &c
 		Pixel pix
		)
{
int dy = y2-y1;
int ady = ((( dy ) >= 0) ? ( dy ) : -( dy )) ;
int dx = x2-x1;
int adx = ((( dx ) >= 0) ? ( dx ) : -( dx )) ;
	if(adx > ady) {
		int x = x1;
		Slope y(y1, y2, adx);
// 		Slope z(z1, z2, adx);
// 		ColorSlope c(c1, c2, adx);
		int xinc = ((( dx ) > 0) ? 1 : -1 ) ;
		do {
// 			ZPixel *zp = &zbuf_y[y][x];
// 			if(z < *zp) {
// 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x] 
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
// 					= cmap->match(c);
 					= pix;
// 			}
// 			++z;
			x += xinc; ++y;
// 			++c;
		} while(x != x2);
	} else if(ady) {
		Slope x(x1, x2, ady);
		int y = y1;
// 		Slope z(z1, z2, ady);
// 		ColorSlope c(c1, c2, ady);
		int yinc = ((( dy ) > 0) ? 1 : -1 ) ;
		do {
// 			ZPixel *zp = &zbuf_y[y][x];
// 			if(z < *zp) {
// 				*zp = z;
ECHO(#ifdef STEREO)
				frame_y[y][x]
ECHO(#else)
				set_mask(1 << (x&3));
				frame_y[y][x/4]
ECHO(#endif)
// 					= cmap->match(c);
 					= pix;
// 			}
// 			++z;
			++x; y += yinc;
// 			++c;
		} while(y != y2);
	} else {
// 		ZPixel *zp = &zbuf_y[y1][x1];
// 		if(z1 < *zp || z2 < *zp) {
// 			if(z1 < z2)
// 				*zp = z1;	
// 			else
// 				*zp = z2;
			set_mask(1 << (x1&3));
// 			frame_y[y1][x1/4] = cmap->match(c1);
// 		frame_y[y1][x1/4] = cmap->match(c);
 			frame_y[y1][x1/4] = pix;
// 		}
	}	
}



 
